from __future__ import annotations

import logging
from collections.abc import Sequence
from typing import TYPE_CHECKING, Any, TypedDict

import torch
from datasets import concatenate_datasets

from mteb._evaluators import ImageTextPairClassificationEvaluator
from mteb.abstasks._statistics_calculation import (
    calculate_image_statistics,
    calculate_text_statistics,
)
from mteb.abstasks.abstask import AbsTask
from mteb.models.models_protocols import EncoderProtocol
from mteb.types.statistics import (
    SplitDescriptiveStatistics,
)

if TYPE_CHECKING:
    from pathlib import Path

    from datasets import Dataset

    from mteb.models.models_protocols import MTEBModels
    from mteb.types import EncodeKwargs
    from mteb.types.statistics import (
        ImageStatistics,
        TextStatistics,
    )

logger = logging.getLogger(__name__)


class ImageTextPairClassificationDescriptiveStatistics(SplitDescriptiveStatistics):
    """Descriptive statistics for ImageTextPairClassification

    Attributes:
        num_samples: number of samples in the dataset.
        text_statistics: Statistics for text
        image_statistics: Statistics for images
    """

    num_samples: int
    text_statistics: TextStatistics
    image_statistics: ImageStatistics


class ImageTextPairClassificationMetrics(TypedDict):
    """ImageTextPairClassification metrics.

    Attributes:
        image_acc: Accuracy of image retrieval.
        text_acc: Accuracy of text retrieval.
        accuracy: Overall accuracy.
    """

    image_acc: float
    text_acc: float
    accuracy: float


class AbsTaskImageTextPairClassification(AbsTask):
    """Abstract class for Image Text Pair Classification tasks (Compositionality evaluation).

    The similarity is computed between pairs and the results are ranked.
    Note that the number of images and the number of captions can be different.

    Attributes:
        dataset: A HuggingFace Dataset containing the data for the ImageTextPairClassification task. Should have columns:
            - images: List of images.
            - captions: List of captions.
        images_column_names: Name of the column(s) containing the images.
        texts_column_names: Name of the column(s) containing the captions.
        abstask_prompt: Prompt to use for the task for instruction model if not prompt is provided in TaskMetadata.prompt.
    """

    # it can be ["image_0", "image_1"]; ["text_0", "text_1"] for datasets like WinoGround
    images_column_names: str | Sequence[str] = "image"
    texts_column_names: str | Sequence[str] = "caption"

    def _calculate_descriptive_statistics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> ImageTextPairClassificationDescriptiveStatistics:
        if compute_overall:
            dataset = concatenate_datasets(
                [
                    self.dataset[hf_subset][split]
                    for hf_subset in self.metadata.eval_langs
                ]
            )
        else:
            dataset = (
                self.dataset[split]
                if hf_subset is None
                else self.dataset[hf_subset][split]
            )
        num_samples = len(dataset)

        images = None
        texts = None

        if isinstance(self.images_column_names, str):
            images = list(dataset[self.images_column_names])
        elif isinstance(self.images_column_names, Sequence):
            images = [
                img
                for img_column in self.images_column_names
                for img in dataset[img_column]
            ]

        if isinstance(self.texts_column_names, str):
            texts = list(dataset[self.texts_column_names])
        elif isinstance(self.texts_column_names, Sequence):
            texts = [
                text
                for text_column in self.texts_column_names
                for text in dataset[text_column]
            ]

        return ImageTextPairClassificationDescriptiveStatistics(
            num_samples=num_samples,
            text_statistics=calculate_text_statistics(texts),
            image_statistics=calculate_image_statistics(images),
        )

    def _evaluate_subset(
        self,
        model: MTEBModels,
        data_split: Dataset,
        *,
        encode_kwargs: EncodeKwargs,
        hf_split: str,
        hf_subset: str,
        prediction_folder: Path | None = None,
        num_proc: int | None = None,
        **kwargs: Any,
    ) -> ImageTextPairClassificationMetrics:
        if not isinstance(model, EncoderProtocol):
            raise TypeError("Expected model to be an instance of EncoderProtocol")
        select_columns = []
        for columns in (self.images_column_names, self.texts_column_names):
            if isinstance(columns, str):
                select_columns.append(columns)
            else:
                select_columns.extend(columns)

        data_split = data_split.select_columns(select_columns)
        num_images_per_sample = (
            1
            if isinstance(self.images_column_names, str)
            else len(self.images_column_names)
        )
        num_texts_per_sample = (
            1
            if isinstance(self.texts_column_names, str)
            else len(self.texts_column_names)
        )
        evaluator = ImageTextPairClassificationEvaluator(
            data_split,
            images_column_names=self.images_column_names,
            texts_column_names=self.texts_column_names,
            task_metadata=self.metadata,
            num_texts_per_sample=num_texts_per_sample,
            num_images_per_sample=num_images_per_sample,
            hf_split=hf_split,
            hf_subset=hf_subset,
            **kwargs,
        )
        scores: list[torch.Tensor] = evaluator(
            model, encode_kwargs=encode_kwargs, num_proc=num_proc
        )  # type: ignore[assignment]
        if prediction_folder:
            self._save_task_predictions(
                [score.tolist() for score in scores],
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )

        return self._compute_metrics(
            scores,
            num_images_per_sample,
            num_texts_per_sample,
        )

    def _compute_metrics(
        self,
        scores: list[torch.Tensor],
        num_images_per_sample: int,
        num_texts_per_sample: int,
    ) -> ImageTextPairClassificationMetrics:
        image_score = []
        text_score = []
        all_correct_scores = []
        img_ground_truths = torch.arange(num_images_per_sample)
        caption_ground_truths = torch.arange(num_texts_per_sample)

        for score in scores:
            image_closest_text = score.argmax(dim=1)  # shape = (num_images_per_sample)
            text_closest_image = score.argmax(dim=0)  # shape = (num_texts_per_sample)
            pred_text_is_correct = (
                (image_closest_text == img_ground_truths).all().item()
            )
            pred_image_is_correct = (
                (text_closest_image == caption_ground_truths).all().item()
            )
            all_correct = pred_text_is_correct and pred_image_is_correct
            image_score.append(pred_image_is_correct)
            text_score.append(pred_text_is_correct)
            all_correct_scores.append(all_correct)

        return ImageTextPairClassificationMetrics(
            image_acc=torch.Tensor(image_score).float().mean().item(),
            text_acc=torch.Tensor(text_score).float().mean().item(),
            accuracy=torch.Tensor(all_correct_scores).float().mean().item(),
        )

    def _push_dataset_to_hub(
        self,
        repo_name: str,
        num_proc: int | None = None,
    ) -> None:
        text_columns = (
            [self.texts_column_names]
            if isinstance(self.texts_column_names, str)
            else self.texts_column_names
        )
        image_columns = (
            [self.images_column_names]
            if isinstance(self.images_column_names, str)
            else self.images_column_names
        )

        self._upload_dataset_to_hub(
            repo_name,
            [*text_columns, *image_columns],
            num_proc=num_proc,
        )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class TenKGnadClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="TenKGnadClassification",
        description="10k German News Articles Dataset (10kGNAD) contains news articles from the online Austrian newspaper website DER Standard with their topic classification (9 classes).",
        reference="https://tblock.github.io/10kGNAD/",
        dataset={
            "path": "mteb/TenKGnadClassification",
            "revision": "ae9862bbcddc27b4bd93e2a7b463b7b5d05c6c55",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["deu-Latn"],
        main_score="accuracy",
        date=("2015-06-01", "2016-05-31"),
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{Schabus2017,
  address = {Tokyo, Japan},
  author = {Dietmar Schabus and Marcin Skowron and Martin Trapp},
  booktitle = {Proceedings of the 40th International ACM SIGIR Conference on Research and Development in Information Retrieval (SIGIR)},
  doi = {10.1145/3077136.3080711},
  month = aug,
  pages = {1241--1244},
  title = {One Million Posts: A Data Set of German Online Discussions},
  year = {2017},
}
""",
        superseded_by="TenKGnadClassification.v2",
    )


class TenKGnadClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="TenKGnadClassification.v2",
        description="10k German News Articles Dataset (10kGNAD) contains news articles from the online Austrian newspaper website DER Standard with their topic classification (9 classes). This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://tblock.github.io/10kGNAD/",
        dataset={
            "path": "mteb/ten_k_gnad",
            "revision": "fc6825fe0d813e7fc92f05fe63ac4bb3ee191c4d",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["deu-Latn"],
        main_score="accuracy",
        date=("2015-06-01", "2016-05-31"),
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{Schabus2017,
  address = {Tokyo, Japan},
  author = {Dietmar Schabus and Marcin Skowron and Martin Trapp},
  booktitle = {Proceedings of the 40th International ACM SIGIR Conference on Research and Development in Information Retrieval (SIGIR)},
  doi = {10.1145/3077136.3080711},
  month = aug,
  pages = {1241--1244},
  title = {One Million Posts: A Data Set of German Online Discussions},
  year = {2017},
}
""",
        adapted_from=["TenKGnadClassification"],
    )



from .klue_mrc_domain_clustering import KlueMrcDomainClustering
from .klue_ynat_mrc_category_clustering import KlueYnatMrcCategoryClustering

__all__ = ["KlueMrcDomainClustering", "KlueYnatMrcCategoryClustering"]



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class SpokeNEnglishClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SpokeNEnglish",
        description="Human Sound Classification Dataset.",
        reference="https://zenodo.org/records/10810044",
        dataset={
            "path": "mteb/SpokeN-100-English",
            "revision": "afbff14d927de14412d8124502313ea6d9d140e0",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2024-01-01", "2024-01-01"),
        domains=["Spoken"],
        task_subtypes=["Vocal Sound Classification"],
        license="cc-by-sa-4.0",
        annotations_creators="LM-generated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@misc{groh2024spoken100crosslingualbenchmarkingdataset,
  archiveprefix = {arXiv},
  author = {René Groh and Nina Goes and Andreas M. Kist},
  eprint = {2403.09753},
  primaryclass = {cs.SD},
  title = {SpokeN-100: A Cross-Lingual Benchmarking Dataset for The Classification of Spoken Numbers in Different Languages},
  url = {https://arxiv.org/abs/2403.09753},
  year = {2024},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "label"

    is_cross_validation: bool = True



from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class OxfordFlowersClassification(AbsTaskClassification):
    input_column_name: str = "image"
    samples_per_label: int = 16
    n_experiments: int = 5

    metadata = TaskMetadata(
        name="OxfordFlowersClassification",
        description="Classifying flowers",
        reference="https://huggingface.co/datasets/nelorth/oxford-flowers/viewer/default/train",
        dataset={
            "path": "mteb/oxford-flowers",
            "revision": "df976877feae05654b0fcd8b394baff731b00652",
        },
        type="ImageClassification",
        category="i2c",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2012-01-01",
            "2015-12-31",
        ),  # Estimated range for the collection of reviews
        domains=["Reviews"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{4756141,
  author = {Nilsback, Maria-Elena and Zisserman, Andrew},
  booktitle = {2008 Sixth Indian Conference on Computer Vision, Graphics & Image Processing},
  doi = {10.1109/ICVGIP.2008.47},
  keywords = {Shape;Kernel;Distributed computing;Support vector machines;Support vector machine classification;object classification;segmentation},
  number = {},
  pages = {722-729},
  title = {Automated Flower Classification over a Large Number of Classes},
  volume = {},
  year = {2008},
}
""",
    )



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.zeroshot_classification import (
    AbsTaskZeroShotClassification,
)


class FER2013ZeroShotClassification(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="FER2013ZeroShot",
        description="Classifying facial emotions.",
        reference="https://arxiv.org/abs/1412.6572",
        dataset={
            "path": "mteb/wds_fer2013",
            "revision": "d9cabab388cb32bc071a1601ef68cb2f70ca0ba7",
        },
        type="ZeroShotClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2014-01-01",
            "2014-12-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Emotion recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image", "text"],
        sample_creation="created",
        bibtex_citation=r"""
@misc{goodfellow2015explainingharnessingadversarialexamples,
  archiveprefix = {arXiv},
  author = {Ian J. Goodfellow and Jonathon Shlens and Christian Szegedy},
  eprint = {1412.6572},
  primaryclass = {stat.ML},
  title = {Explaining and Harnessing Adversarial Examples},
  url = {https://arxiv.org/abs/1412.6572},
  year = {2015},
}
""",
    )
    input_column_name: str = "jpg"
    label_column_name: str = "cls"

    def get_candidate_labels(self) -> list[str]:
        labels = ["angry", "disgust", "fear", "happy", "neutral", "sad", "surprise"]
        return [f"a photo of a {name} looking face." for name in labels]



from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from datasets import DatasetDict

from mteb.abstasks._data_filter.filters import (
    deduplicate,
    filter_empty,
    filter_short,
    filter_train_leakage,
    filter_unclear_label,
    split_train_test,
)

if TYPE_CHECKING:
    from mteb import TaskMetadata
    from mteb.abstasks import AbsTaskClassification

logger = logging.getLogger(__name__)


def clean_dataset(
    ds: DatasetDict,
    metadata: TaskMetadata,
    train_split: str,
    input_column: str,
    label_column: str,
    subset: str | None = None,
) -> DatasetDict:
    """Apply the full cleaning pipeline with logging."""
    logger.info("[clean_dataset] Starting dataset cleaning pipeline...")

    transforms = [
        ("filter_empty", filter_empty),
        ("deduplicate", deduplicate),
    ]

    skip_cjk_codes = {"zho", "jpn", "tha", "mya", "cmn"}
    logger.info("[clean_dataset] Applying short-text filter")
    cur_langs = (
        metadata.eval_langs[subset]
        if isinstance(metadata.eval_langs, dict) and subset
        else metadata.eval_langs
    )
    apply_short = not any(lang.split("-")[0] in skip_cjk_codes for lang in cur_langs)
    if apply_short:
        logger.info("[clean_dataset] Applying short-text filter")
        transforms.append(("filter_short", filter_short))

    for split in [train_split, *metadata.eval_splits]:
        if split not in ds:
            logger.warning(f"[clean_dataset] Split '{split}' missing; skipping.")
            continue

        for name, fn in transforms:
            before = len(ds[split])
            ds[split] = fn(ds[split], input_column=input_column)
            logger.info(
                f"[clean_dataset:{split}] {name} removed={before - len(ds[split])}"
            )

    ds = split_train_test(ds, metadata, train_split, label_column)

    for split in metadata.eval_splits:
        if split == train_split:
            continue
        before = len(ds[split])
        ds[split] = filter_train_leakage(ds[train_split], ds[split], input_column)
        logger.info(
            f"[clean_dataset:{split}] leakage_removed={before - len(ds[split])}"
        )

    ds = filter_unclear_label(ds, input_column=input_column, label_column=label_column)

    logger.info("[clean_dataset] Cleaning pipeline complete.")
    return ds


def process_classification(
    task: AbsTaskClassification,
) -> DatasetDict | dict[str, DatasetDict]:
    """Process classification task dataset(s) with cleaning pipeline."""
    if not task.data_loaded:
        task.load_data()
    if isinstance(task.dataset, DatasetDict):
        return clean_dataset(
            task.dataset,
            task.metadata,
            task.train_split,
            task.input_column_name,
            task.label_column_name,
            subset=None,
        )

    if task.dataset is None:
        raise ValueError("Task dataset is None.")

    new_ds = {}
    for subset in task.dataset:
        new_ds[subset] = clean_dataset(
            task.dataset[subset],
            task.metadata,
            task.train_split,
            task.input_column_name,
            task.label_column_name,
            subset=subset,
        )
    return new_ds



from .macedonian_tweet_sentiment_classification import (
    MacedonianTweetSentimentClassification,
    MacedonianTweetSentimentClassificationV2,
)

__all__ = [
    "MacedonianTweetSentimentClassification",
    "MacedonianTweetSentimentClassificationV2",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class Touche2020NL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="Touche2020-NL",
        description="Touché Task 1: Argument Retrieval for Controversial Questions. Touche2020-NL is a Dutch translation.",
        reference="https://huggingface.co/datasets/clips/beir-nl-webis-touche2020",
        dataset={
            "path": "clips/beir-nl-webis-touche2020",
            "revision": "b69e63cbe72c5ce0489f69e88c35f51f13a3f993",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="ndcg_at_10",
        date=("2020-09-23", "2020-09-23"),
        domains=["Academic", "Non-fiction"],
        task_subtypes=["Question answering"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated and verified",  # manually checked a small subset
        bibtex_citation=r"""
@misc{banar2024beirnlzeroshotinformationretrieval,
  archiveprefix = {arXiv},
  author = {Nikolay Banar and Ehsan Lotfi and Walter Daelemans},
  eprint = {2412.08329},
  primaryclass = {cs.CL},
  title = {BEIR-NL: Zero-shot Information Retrieval Benchmark for the Dutch Language},
  url = {https://arxiv.org/abs/2412.08329},
  year = {2024},
}
""",
        adapted_from=["Touche2020"],
    )



from mteb.models.model_meta import ModelMeta
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

Euler_Legal_Embedding_V1 = ModelMeta(
    loader=sentence_transformers_loader,
    name="Mira190/Euler-Legal-Embedding-V1",
    model_type=["dense"],
    revision="df607ed9e25e569514a99c27cdaaab16e76b6dd4",
    release_date="2025-11-06",
    languages=["eng-Latn"],
    n_parameters=7567295488,
    n_embedding_parameters=621219840,
    memory_usage_mb=15618,
    max_tokens=1536,
    embed_dim=4096,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/Mira190/Euler-Legal-Embedding-V1",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=set(),  # final-data-new-anonymized-grok4-filtered
    adapted_from="Qwen/Qwen3-Embedding-8B",
    superseded_by=None,
    citation="""@misc{euler2025legal,
      title={Euler-Legal-Embedding: Advanced Legal Representation Learning},
      author={LawRank Team},
      year={2025},
      publisher={Hugging Face}
}""",
)



from __future__ import annotations

import logging
import tempfile
from pathlib import Path
from typing import TYPE_CHECKING, Any

import numpy as np
import torch
from tqdm.auto import tqdm

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_package
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput, TextInput

logger = logging.getLogger(__name__)


class MSClapWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str = "microsoft/msclap-2023",
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_s: float = 30.0,
        **kwargs: Any,
    ):
        requires_package(
            self,
            "msclap",
            "pip install 'mteb[msclap]'",
        )
        from msclap import CLAP

        self.model_name = model_name
        self.device = device
        self.sampling_rate = 48000
        self.max_audio_length_s = max_audio_length_s

        if "2022" in self.model_name:
            self.version = "2022"
            self.text_length = 100
        elif "2023" in self.model_name:
            self.version = "2023"
            self.text_length = 77
        else:
            self.version = "2023"
            self.text_length = 77

        self.use_cuda = device == "cuda"
        self.model = CLAP(version=self.version, use_cuda=self.use_cuda)
        self.model.clap = self.model.clap.to(self.device)
        self.tokenizer = self.model.tokenizer

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> np.ndarray:
        import soundfile as sf

        inputs.collate_fn = AudioCollator(self.sampling_rate)

        all_embeddings = []
        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            temp_files = []
            audio_arrays = [audio["array"] for audio in batch["audio"]]

            try:
                for array in audio_arrays:
                    # Write to temp file - msclap expects file paths
                    temp_file = tempfile.NamedTemporaryFile(suffix=".wav", delete=False)
                    temp_files.append(temp_file.name)
                    sf.write(temp_file.name, array, self.sampling_rate)

                with torch.no_grad():
                    # Use the official msclap API that expects file paths
                    # https://github.com/microsoft/CLAP#api
                    audio_features = self.model.get_audio_embeddings(
                        temp_files, resample=False
                    )
                    # Normalize embeddings
                    audio_features = audio_features / audio_features.norm(
                        dim=-1, keepdim=True
                    )
                    all_embeddings.append(audio_features.cpu().detach().numpy())
            finally:
                # Clean up temp files

                for f in temp_files:
                    try:
                        Path(f).unlink()
                    except OSError:
                        pass

        return np.vstack(all_embeddings)

    def get_text_embeddings(
        self,
        inputs: DataLoader[TextInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> np.ndarray:
        text_embeddings = []
        for batch in tqdm(
            inputs, disable=not show_progress_bar, desc="Processing text batches"
        ):
            texts = batch["text"]

            features = self.tokenizer(
                texts,
                return_tensors="pt",
                padding="max_length",
                truncation=True,
                max_length=self.text_length,
            )
            features = {k: v.to(self.device) for k, v in features.items()}

            with torch.no_grad():
                text_features = self.model.clap.caption_encoder(features)
                text_features = text_features / text_features.norm(dim=-1, keepdim=True)
            text_embeddings.append(text_features.cpu().detach().numpy())

        return np.vstack(text_embeddings)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        audio_embeddings = None

        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "audio" in inputs.dataset.features:
            audio_embeddings = self.get_audio_embeddings(inputs, **kwargs)

        if text_embeddings is not None and audio_embeddings is not None:
            if len(text_embeddings) != len(audio_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + audio_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif audio_embeddings is not None:
            return audio_embeddings
        raise ValueError


# Microsoft CLAP Model metadata
ms_clap_2022 = ModelMeta(
    loader=MSClapWrapper,
    name="microsoft/msclap-2022",
    languages=["eng-Latn"],
    revision="no_revision",
    release_date="2022-12-01",
    modalities=["audio", "text"],
    n_parameters=196_000_000,
    memory_usage_mb=750,
    max_tokens=None,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code="https://github.com/microsoft/CLAP",
    public_training_data="https://github.com/microsoft/CLAP",
    framework=["PyTorch"],
    reference="https://github.com/microsoft/CLAP",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=set(),
    citation="""
@inproceedings{CLAP2022,
  title={Clap learning audio concepts from natural language supervision},
  author={Elizalde, Benjamin and Deshmukh, Soham and Al Ismail, Mahmoud and Wang, Huaming},
  booktitle={ICASSP 2023-2023 IEEE International Conference on Acoustics, Speech and Signal Processing (ICASSP)},
  pages={1--5},
  year={2023},
  organization={IEEE}
}
""",
)

ms_clap_2023 = ModelMeta(
    loader=MSClapWrapper,
    name="microsoft/msclap-2023",
    languages=["eng-Latn"],
    revision="no_revision",
    release_date="2023-09-01",
    modalities=["audio", "text"],
    n_parameters=160_000_000,
    memory_usage_mb=610,
    max_tokens=None,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code="https://github.com/microsoft/CLAP",
    public_training_data="https://github.com/microsoft/CLAP",
    framework=["PyTorch"],
    reference="https://github.com/microsoft/CLAP",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=set(),
    citation="""
@misc{CLAP2023,
      title={Natural Language Supervision for General-Purpose Audio Representations},
      author={Benjamin Elizalde and Soham Deshmukh and Huaming Wang},
      year={2023},
      eprint={2309.05767},
      archivePrefix={arXiv},
      primaryClass={cs.SD},
      url={https://arxiv.org/abs/2309.05767}
}
""",
)



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "da": ["dan-Latn"],
    "en": ["eng-Latn"],
    "fi": ["fin-Latn"],
    "no": ["nob-Latn"],
    "sv": ["swe-Latn"],
}


class ScandiSentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="ScandiSentClassification",
        dataset={
            "path": "mteb/scandisent",
            "revision": "97672414ad7453a106edfbfb1a0ceb152355b9dd",
        },
        description="The corpus is crawled from se.trustpilot.com, no.trustpilot.com, dk.trustpilot.com, fi.trustpilot.com and trustpilot.com.",
        reference="https://github.com/timpal0l/ScandiSent",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="accuracy",
        date=("2020-09-01", "2022-04-09"),
        domains=["Reviews", "Written"],
        dialect=[],
        task_subtypes=["Sentiment/Hate speech"],
        license="openrail",
        annotations_creators="expert-annotated",
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{isbister-etal-2021-stop,
  address = {Reykjavik, Iceland (Online)},
  author = {Isbister, Tim  and
Carlsson, Fredrik  and
Sahlgren, Magnus},
  booktitle = {Proceedings of the 23rd Nordic Conference on Computational Linguistics (NoDaLiDa)},
  editor = {Dobnik, Simon  and
{\O}vrelid, Lilja},
  month = may # { 31--2 } # jun,
  pages = {385--390},
  publisher = {Link{\"o}ping University Electronic Press, Sweden},
  title = {Should we Stop Training More Monolingual Models, and Simply Use Machine Translation Instead?},
  url = {https://aclanthology.org/2021.nodalida-main.42/},
  year = {2021},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class MultiSWEbenchReranking(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MultiSWEbenchRR",
        description="Multilingual Software Issue Localization.",
        reference="https://multi-swe-bench.github.io/#/",
        dataset={
            "path": "mteb/MultiSWEbenchRR",
            "revision": "f80517681e98077bb9b20470ab562bb6e94e7897",
        },
        type="Reranking",
        category="t2t",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["eng-Latn", "python-Code"],
        main_score="recall_at_10",
        date=("2023-10-10", "2023-10-10"),
        domains=["Programming", "Written"],
        task_subtypes=["Code retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        prompt={
            "query": "Given a github issue, identify the code that needs to be changed to fix the issue."
        },
        bibtex_citation=r"""
@misc{zan2025multiswebench,
  archiveprefix = {arXiv},
  author = {Daoguang Zan and Zhirong Huang and Wei Liu and Hanwu Chen and Linhao Zhang and Shulin Xin and Lu Chen and Qi Liu and Xiaojian Zhong and Aoyan Li and Siyao Liu and Yongsheng Xiao and Liangqiang Chen and Yuyu Zhang and Jing Su and Tianyu Liu and Rui Long and Kai Shen and Liang Xiang},
  eprint = {2504.02605},
  primaryclass = {cs.SE},
  title = {Multi-SWE-bench: A Multilingual Benchmark for Issue Resolving},
  url = {https://arxiv.org/abs/2504.02605},
  year = {2025},
}
""",
    )



from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata

EVAL_LANGS_MAP = {
    "afr_Latn": ["afr-Latn"],  # Afrikaans
    "amh_Ethi": ["amh-Ethi"],  # Amharic
    "arb_Arab": ["arb-Arab"],  # Arabic
    "asm_Beng": ["asm-Beng"],  # Assamese
    "ast_Latn": ["ast-Latn"],  # Asturian
    "azj_Latn": ["azj-Latn"],  # South Azerbaijani
    "bel_Cyrl": ["bel-Cyrl"],  # Belarusian
    "bul_Cyrl": ["bul-Cyrl"],  # Bulgarian
    "ben_Beng": ["ben-Beng"],  # Bengali
    "bos_Latn": ["bos-Latn"],  # Bosnian
    "cat_Latn": ["cat-Latn"],  # Catalan
    "ceb_Latn": ["ceb-Latn"],  # Cebuano
    "ckb_Arab": ["ckb-Arab"],  # Central Kurdish
    "zho_Hans": ["zho-Hans"],  # Chinese (Simplified)
    "ces_Latn": ["ces-Latn"],  # Czech
    "cym_Latn": ["cym-Latn"],  # Welsh
    "dan_Latn": ["dan-Latn"],  # Danish
    "deu_Latn": ["deu-Latn"],  # German
    "ell_Grek": ["ell-Grek"],  # Greek
    "eng_Latn": ["eng-Latn"],  # English
    "spa_Latn": ["spa-Latn"],  # Spanish
    "est_Latn": ["est-Latn"],  # Estonian
    "pes_Arab": ["pes-Arab"],  # Persian
    "fin_Latn": ["fin-Latn"],  # Finnish
    "tgl_Latn": ["tgl-Latn"],  # Tagalog
    "fra_Latn": ["fra-Latn"],  # French
    "gle_Latn": ["gle-Latn"],  # Irish
    "glg_Latn": ["glg-Latn"],  # Galician
    "guj_Gujr": ["guj-Gujr"],  # Gujarati
    "hau_Latn": ["hau-Latn"],  # Hausa
    "heb_Hebr": ["heb-Hebr"],  # Hebrew
    "hin_Deva": ["hin-Deva"],  # Hindi
    "hrv_Latn": ["hrv-Latn"],  # Croatian
    "hun_Latn": ["hun-Latn"],  # Hungarian
    "hye_Armn": ["hye-Armn"],  # Armenian
    "ind_Latn": ["ind-Latn"],  # Indonesian
    "ibo_Latn": ["ibo-Latn"],  # Igbo
    "isl_Latn": ["isl-Latn"],  # Icelandic
    "ita_Latn": ["ita-Latn"],  # Italian
    "jpn_Jpan": ["jpn-Jpan"],  # Japanese
    "jav_Latn": ["jav-Latn"],  # Javanese
    "kat_Geor": ["kat-Geor"],  # Georgian
    "kam_Latn": ["kam-Latn"],  # Kamba
    "kea_Latn": ["kea-Latn"],  # Kabuverdianu
    "kaz_Cyrl": ["kaz-Cyrl"],  # Kazakh
    "khm_Khmr": ["khm-Khmr"],  # Khmer
    "kan_Knda": ["kan-Knda"],  # Kannada
    "kor_Hang": ["kor-Hang"],  # Korean
    "kir_Cyrl": ["kir-Cyrl"],  # Kyrgyz
    "ltz_Latn": ["ltz-Latn"],  # Luxembourgish
    "lug_Latn": ["lug-Latn"],  # Ganda
    "lin_Latn": ["lin-Latn"],  # Lingala
    "lao_Laoo": ["lao-Laoo"],  # Lao
    "lit_Latn": ["lit-Latn"],  # Lithuanian
    "luo_Latn": ["luo-Latn"],  # Luo
    "lvs_Latn": ["lvs-Latn"],  # Latvian
    "mri_Latn": ["mri-Latn"],  # Maori
    "mkd_Cyrl": ["mkd-Cyrl"],  # Macedonian
    "mal_Mlym": ["mal-Mlym"],  # Malayalam
    "khk_Cyrl": ["khk-Cyrl"],  # Mongolian (Halh)
    "mar_Deva": ["mar-Deva"],  # Marathi
    "zsm_Latn": ["zsm-Latn"],  # Malay
    "mlt_Latn": ["mlt-Latn"],  # Maltese
    "mya_Mymr": ["mya-Mymr"],  # Burmese
    "nob_Latn": ["nob-Latn"],  # Norwegian Bokmål
    "npi_Deva": ["npi-Deva"],  # Nepali
    "nld_Latn": ["nld-Latn"],  # Dutch
    "nso_Latn": ["nso-Latn"],  # Northern Sotho
    "nya_Latn": ["nya-Latn"],  # Chichewa
    "oci_Latn": ["oci-Latn"],  # Occitan
    "ory_Orya": ["ory-Orya"],  # Odia
    "pan_Guru": ["pan-Guru"],  # Punjabi
    "pol_Latn": ["pol-Latn"],  # Polish
    "pbt_Arab": ["pbt-Arab"],  # Pashto
    "por_Latn": ["por-Latn"],  # Portuguese
    "ron_Latn": ["ron-Latn"],  # Romanian
    "rus_Cyrl": ["rus-Cyrl"],  # Russian
    "snd_Arab": ["snd-Arab"],  # Sindhi
    "slk_Latn": ["slk-Latn"],  # Slovak
    "slv_Latn": ["slv-Latn"],  # Slovenian
    "sna_Latn": ["sna-Latn"],  # Shona
    "som_Latn": ["som-Latn"],  # Somali
    "srp_Cyrl": ["srp-Cyrl"],  # Serbian
    "swe_Latn": ["swe-Latn"],  # Swedish
    "swh_Latn": ["swh-Latn"],  # Swahili
    "tam_Taml": ["tam-Taml"],  # Tamil
    "tel_Telu": ["tel-Telu"],  # Telugu
    "tgk_Cyrl": ["tgk-Cyrl"],  # Tajik
    "tha_Thai": ["tha-Thai"],  # Thai
    "tur_Latn": ["tur-Latn"],  # Turkish
    "ukr_Cyrl": ["ukr-Cyrl"],  # Ukrainian
    "umb_Latn": ["umb-Latn"],  # Umbundu
    "urd_Arab": ["urd-Arab"],  # Urdu
    "uzn_Latn": ["uzn-Latn"],  # Northern Uzbek
    "vie_Latn": ["vie-Latn"],  # Vietnamese
    "wol_Latn": ["wol-Latn"],  # Wolof
    "xho_Latn": ["xho-Latn"],  # Xhosa
    "yor_Latn": ["yor-Latn"],  # Yoruba
    "zho_Hant": ["zho-Hant"],  # Chinese (Traditional)
    "zul_Latn": ["zul-Latn"],  # Zulu
    "fuv_Latn": ["fuv-Latn"],  # Fulfulde
    "gaz_Latn": ["gaz-Latn"],  # Lower Sorbian
}


class SIBFLEURSMultilingualClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SIBFLEURS",
        description="Topic Classification for multilingual audio dataset. This dataset is a stratified and downsampled subset of the SIBFLEURS dataset, which is a collection of 1000+ hours of audio data in 100+ languages.",
        reference="https://huggingface.co/datasets/WueNLP/sib-fleurs",
        dataset={
            "path": "mteb/sib-fleurs-multilingual-mini",
            "revision": "186b61175fbd77059f769b6bf1110d449a2ff311",
        },
        type="AudioMultilabelClassification",
        category="a2c",
        eval_splits=["train"],
        eval_langs=EVAL_LANGS_MAP,
        main_score="accuracy",
        date=(
            "2024-12-09",
            "2024-12-13",
        ),
        domains=[
            "Encyclopaedic"
        ],  # original FLORES-101 dataset is read-out wikipedia corpus
        task_subtypes=["Topic classification"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@misc{schmidt2025fleursslumassivelymultilingualbenchmark,
  archiveprefix = {arXiv},
  author = {Fabian David Schmidt and Ivan Vulić and Goran Glavaš and David Ifeoluwa Adelani},
  eprint = {2501.06117},
  primaryclass = {cs.CL},
  title = {Fleurs-SLU: A Massively Multilingual Benchmark for Spoken Language Understanding},
  url = {https://arxiv.org/abs/2501.06117},
  year = {2025},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "category"
    is_cross_validation: bool = True



from mteb.abstasks import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class CQADupstackWordpressRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-Wordpress-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-wordpress-pl",
        dataset={
            "path": "mteb/CQADupstack-Wordpress-PL",
            "revision": "b93d34a19cea03d6c7b00e261eccace6b29535ee",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Written", "Web", "Programming"],
        task_subtypes=["Question answering"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackWordpressRetrieval"],
    )


class CQADupstackWebmastersRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-Webmasters-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-webmasters-pl",
        dataset={
            "path": "mteb/CQADupstack-Webmasters-PL",
            "revision": "aca4f7fb71a6d367e1c8f5e3fda118a4bcdb41d3",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Written", "Web"],
        task_subtypes=["Question answering"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackWebmastersRetrieval"],
    )


class CQADupstackUnixRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-Unix-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-unix-pl",
        dataset={
            "path": "mteb/CQADupstack-Unix-PL",
            "revision": "f41e1fefa91ccb25964afb24e193d1e2a14bb4a3",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Written", "Web", "Programming"],
        task_subtypes=["Question answering", "Duplicate Detection"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackUnixRetrieval"],
    )


class CQADupstackTexRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-Tex-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-tex-pl",
        dataset={
            "path": "mteb/CQADupstack-Tex-PL",
            "revision": "7e24a1d7543917b655899680f7e03fed3fe967d4",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Written", "Non-fiction"],
        task_subtypes=["Question answering", "Duplicate Detection"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackTexRetrieval"],
    )


class CQADupstackStatsRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-Stats-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-stats-pl",
        dataset={
            "path": "mteb/CQADupstack-Stats-PL",
            "revision": "c1fac8ba4c0f1ed59910f5278c6bf071008b0180",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Written", "Academic", "Non-fiction"],
        task_subtypes=["Question answering", "Duplicate Detection"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackStatsRetrieval"],
    )


class CQADupstackProgrammersRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-Programmers-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-programmers-pl",
        dataset={
            "path": "mteb/CQADupstack-Programmers-PL",
            "revision": "6536329882b0f06bf6406fdafedc434603fe8ed9",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Programming", "Written", "Non-fiction"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackProgrammersRetrieval"],
    )


class CQADupstackPhysicsRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-Physics-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-physics-pl",
        dataset={
            "path": "mteb/CQADupstack-Physics-PL",
            "revision": "a61c9ba627e0fa44b85578812577e6665a60661c",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Written", "Academic", "Non-fiction"],
        task_subtypes=["Question answering", "Duplicate Detection"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackPhysicsRetrieval"],
    )


class CQADupstackMathematicaRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-Mathematica-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-mathematica-pl",
        dataset={
            "path": "mteb/CQADupstack-Mathematica-PL",
            "revision": "f8c9c441534d4823bf9b06573e239841900dec3e",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Written", "Academic", "Non-fiction"],
        task_subtypes=["Question answering", "Duplicate Detection"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackMathematicaRetrieval"],
    )


class CQADupstackGisRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-Gis-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-gis-pl",
        dataset={
            "path": "mteb/CQADupstack-Gis-PL",
            "revision": "784afb42ad5c0f2e71630644a32f08645bbc87fa",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Written", "Academic", "Non-fiction"],
        task_subtypes=["Question answering", "Duplicate Detection"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackGisRetrieval"],
    )


class CQADupstackGamingRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-Gaming-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-gaming-pl",
        dataset={
            "path": "mteb/CQADupstack-Gaming-PL",
            "revision": "7ae0bc1b9a8a1d5a8587a464e25c853552302ce7",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Web", "Written"],
        task_subtypes=["Question answering", "Duplicate Detection"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackGamingRetrieval"],
    )


class CQADupstackEnglishRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-English-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-english-pl",
        dataset={
            "path": "mteb/CQADupstack-English-PL",
            "revision": "27402a7ed3c8a51f45f2a84cb7dfaf6ab4729dad",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Written"],
        task_subtypes=["Question answering", "Duplicate Detection"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackEnglishRetrieval"],
    )


class CQADupstackAndroidRetrievalPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CQADupstack-Android-PL",
        description="CQADupStack: A Stack Exchange Question Duplicate Pairs Dataset",
        reference="https://huggingface.co/datasets/clarin-knext/cqadupstack-android-pl",
        dataset={
            "path": "mteb/CQADupstack-Android-PL",
            "revision": "1f7d469f28e94c23164476fc4115f14fb3b95a6c",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date,
        domains=["Programming", "Web", "Written", "Non-fiction"],
        task_subtypes=["Question answering", "Duplicate Detection"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["CQADupstackAndroidRetrieval"],
    )



from mteb.abstasks.pair_classification import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata


class PubChemAISentenceParaphrasePC(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="PubChemAISentenceParaphrasePC",
        description="ChemTEB evaluates the performance of text embedding models on chemical domain data.",
        reference="https://arxiv.org/abs/2412.00532",
        dataset={
            "path": "BASF-AI/PubChemAISentenceParaphrasePC",
            "revision": "f33a205966ce032f957c3a22f4f9e378f89a2c56",
        },
        type="PairClassification",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="max_ap",
        date=("2024-06-01", "2024-11-30"),
        domains=["Chemistry"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="LM-generated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@article{kasmaee2024chemteb,
  author = {Kasmaee, Ali Shiraee and Khodadad, Mohammad and Saloot, Mohammad Arshi and Sherck, Nick and Dokas, Stephen and Mahyar, Hamidreza and Samiee, Soheila},
  journal = {arXiv preprint arXiv:2412.00532},
  title = {ChemTEB: Chemical Text Embedding Benchmark, an Overview of Embedding Models Performance \& Efficiency on a Specific Domain},
  year = {2024},
}

@article{kim2023pubchem,
  author = {Kim, Sunghwan and Chen, Jie and Cheng, Tiejun and Gindulyte, Asta and He, Jia and He, Siqian and Li, Qingliang and Shoemaker, Benjamin A and Thiessen, Paul A and Yu, Bo and others},
  journal = {Nucleic acids research},
  number = {D1},
  pages = {D1373--D1380},
  publisher = {Oxford University Press},
  title = {PubChem 2023 update},
  volume = {51},
  year = {2023},
}
""",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        _dataset = {}
        for split in self.metadata.eval_splits:
            hf_dataset = self.dataset[split]
            _dataset[split] = [
                {
                    "sentence1": hf_dataset["sent1"],
                    "sentence2": hf_dataset["sent2"],
                    "labels": hf_dataset["labels"],
                }
            ]
        self.dataset = _dataset



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class AJGT(AbsTaskClassification):
    metadata = TaskMetadata(
        name="AJGT",
        dataset={
            "path": "komari6/ajgt_twitter_ar",
            "revision": "af3f2fa5462ac461b696cb300d66e07ad366057f",
        },
        description="Arabic Jordanian General Tweets (AJGT) Corpus consisted of 1,800 tweets annotated as positive and negative. Modern Standard Arabic (MSA) or Jordanian dialect.",
        reference="https://link.springer.com/chapter/10.1007/978-3-319-60042-0_66/",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["ara-Arab"],
        main_score="accuracy",
        date=("2021-01-01", "2022-01-25"),
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="afl-3.0",
        annotations_creators="human-annotated",
        dialect=["ara-arab-MSA", "ara-arab-JO"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{alomari2017arabic,
  author = {Alomari, Khaled Mohammad and ElSherif, Hatem M and Shaalan, Khaled},
  booktitle = {International Conference on Industrial, Engineering and Other Applications of Applied Intelligent Systems},
  organization = {Springer},
  pages = {602--610},
  title = {Arabic tweets sentimental analysis using machine learning},
  year = {2017},
}
""",
        superseded_by="AJGT.v2",
    )


class AJGTV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="AJGT.v2",
        dataset={
            "path": "mteb/ajgt",
            "revision": "0a3dea7301ee0c051891f04d32f3e8577a9eae36",
        },
        description="Arabic Jordanian General Tweets (AJGT) Corpus consisted of 1,800 tweets (900 for training and 900 for testing) annotated as positive and negative. Modern Standard Arabic (MSA) or Jordanian dialect. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2632)",
        reference="https://link.springer.com/chapter/10.1007/978-3-319-60042-0_66/",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ara-Arab"],
        main_score="accuracy",
        date=("2021-01-01", "2022-01-25"),
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="afl-3.0",
        annotations_creators="human-annotated",
        dialect=["ara-arab-MSA", "ara-arab-JO"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{alomari2017arabic,
  author = {Alomari, Khaled Mohammad and ElSherif, Hatem M and Shaalan, Khaled},
  booktitle = {International Conference on Industrial, Engineering and Other Applications of Applied Intelligent Systems},
  organization = {Springer},
  pages = {602--610},
  title = {Arabic tweets sentimental analysis using machine learning},
  year = {2017},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class OKVQAIT2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="OKVQAIT2TRetrieval",
        description="Retrieval a Wiki passage to answer query about an image.",
        reference="https://okvqa.allenai.org",
        dataset={
            "path": "izhx/UMRB-OKVQA",
            "revision": "96a84a043f5465893670cf616f90e64086c0417a",
        },
        type="Any2AnyRetrieval",
        category="it2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="hit_rate_at_10",
        date=("2019-01-01", "2020-07-29"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["image", "text"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{marino2019ok,
  author = {Marino, Kenneth and Rastegari, Mohammad and Farhadi, Ali and Mottaghi, Roozbeh},
  booktitle = {Proceedings of the IEEE/cvf conference on computer vision and pattern recognition},
  pages = {3195--3204},
  title = {Ok-vqa: A visual question answering benchmark requiring external knowledge},
  year = {2019},
}
""",
        prompt={
            "query": "Retrieve documents that provide an answer to the question alongside the image."
        },
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm
from transformers import UniSpeechSatForCTC, Wav2Vec2FeatureExtractor

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_audio_dependencies
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput


class UniSpeechWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_seconds: float = 30.0,
        **kwargs: Any,
    ):
        requires_audio_dependencies()
        self.model_name = model_name
        self.device = device
        self.max_audio_length_seconds = max_audio_length_seconds

        # UniSpeech uses the same feature extractor as Wav2Vec2
        self.feature_extractor = Wav2Vec2FeatureExtractor.from_pretrained(
            model_name, revision=revision
        )
        self.model = UniSpeechSatForCTC.from_pretrained(
            model_name, revision=revision
        ).to(self.device)
        self.model.eval()
        self.sampling_rate = self.feature_extractor.sampling_rate

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        inputs.collate_fn = AudioCollator(self.sampling_rate)

        all_embeddings = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            audio_arrays = [audio["array"] for audio in batch["audio"]]

            feature_inputs = self.feature_extractor(
                audio_arrays,
                sampling_rate=self.sampling_rate,
                return_tensors="pt",
                padding="longest",
                truncation=True,
                max_length=int(self.max_audio_length_seconds * self.sampling_rate),
                return_attention_mask=True,
            ).to(self.device)

            with torch.no_grad():
                # UniSpeech model outputs
                outputs = self.model(
                    feature_inputs.input_values,
                    attention_mask=feature_inputs.attention_mask,
                    output_hidden_states=True,
                )

                # Get embeddings from last hidden state
                last_hidden_state = outputs.hidden_states[-1]

                # Apply attention-masked pooling to exclude padding tokens
                batch_size, hidden_seq_len, hidden_size = last_hidden_state.shape
                device = last_hidden_state.device

                # Calculate proper hidden lengths based on input attention mask
                input_lengths = feature_inputs.attention_mask.sum(dim=1)
                downsample_ratio = feature_inputs.input_values.shape[1] / hidden_seq_len
                hidden_lengths = (input_lengths.float() / downsample_ratio).long()
                hidden_lengths = torch.clamp(hidden_lengths, min=0, max=hidden_seq_len)

                # Create attention mask for hidden states
                seq_range = torch.arange(hidden_seq_len, device=device).unsqueeze(0)
                hidden_attention_mask = (seq_range < hidden_lengths.unsqueeze(1)).to(
                    last_hidden_state.dtype
                )

                # Apply masked mean pooling
                hidden_attention_mask = hidden_attention_mask.unsqueeze(-1)
                masked_embeddings = last_hidden_state * hidden_attention_mask
                valid_tokens = hidden_attention_mask.sum(dim=1)
                embeddings = masked_embeddings.sum(dim=1) / valid_tokens.clamp(min=1e-9)

                all_embeddings.append(embeddings.cpu().detach())

        return torch.cat(all_embeddings, dim=0).numpy()

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if "audio" not in inputs.dataset.features:
            raise ValueError("UniSpeechWrapper only supports audio inputs.")
        return self.get_audio_embeddings(inputs, **kwargs)


unispeech_base = ModelMeta(
    loader=UniSpeechWrapper,
    name="microsoft/unispeech-sat-base-100h-libri-ft",
    languages=["eng-Latn"],
    open_weights=True,
    revision="b294556d8d77cc539f9d08bef0ddbb0f60985328",
    release_date="2021-10-12",
    max_tokens=float("inf"),
    n_parameters=94_000_000,
    memory_usage_mb=359,
    embed_dim=768,
    license="apache-2.0",
    reference="https://huggingface.co/microsoft/unispeech-sat-base-100h-libri-ft",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={"LibriSpeech"},
    modalities=["audio"],
    citation="""
@misc{chen2021unispeechsatuniversalspeechrepresentation,
      title={UniSpeech-SAT: Universal Speech Representation Learning with Speaker Aware Pre-Training},
      author={Sanyuan Chen and Yu Wu and Chengyi Wang and Zhengyang Chen and Zhuo Chen and Shujie Liu and Jian Wu and Yao Qian and Furu Wei and Jinyu Li and Xiangzhan Yu},
      year={2021},
      eprint={2110.05752},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2110.05752},
}""",
)



from __future__ import annotations

import hashlib
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from collections.abc import Mapping

    from PIL import Image


def _hash_item(item: Mapping[str, Any]) -> str:
    item_hash = ""
    if "text" in item:
        item_text: str = item["text"]
        item_hash = hashlib.sha256(item_text.encode()).hexdigest()

    if "image" in item:
        image: Image.Image = item["image"]
        item_hash += hashlib.sha256(image.tobytes()).hexdigest()

    if item_hash == 0:
        raise TypeError(f"Unsupported cache key type: {type(item)}")

    return item_hash



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class KoVidore2CybersecurityRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="KoVidore2CybersecurityRetrieval",
        description="Retrieve associated pages according to questions. This dataset, Cybersecurity, is a corpus of technical reports on cyber threat trends and security incident responses in Korea, intended for complex-document understanding tasks.",
        reference="https://github.com/whybe-choi/kovidore-data-generator",
        dataset={
            "path": "whybe-choi/kovidore-v2-cybersecurity-mteb",
            "revision": "577d7c45f79d8eb4e7584db3990f91daa7e47956",
        },
        type="DocumentUnderstanding",
        category="t2i",
        eval_splits=["test"],
        eval_langs=["kor-Hang"],
        main_score="ndcg_at_10",
        date=("2025-12-21", "2026-01-06"),
        domains=["Social"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation="""
@misc{choi2026kovidorev2,
  author = {Yongbin Choi},
  note = {A benchmark for evaluating Korean vision document retrieval with multi-page reasoning queries in practical domains},
  title = {KoViDoRe v2: a comprehensive evaluation of vision document retrieval for enterprise use-cases},
  url = {https://github.com/whybe-choi/kovidore-data-generator},
  year = {2026},
}
""",
        prompt={"query": "Find a screenshot that is relevant to the user's question."},
    )


class KoVidore2EconomicRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="KoVidore2EconomicRetrieval",
        description="Retrieve associated pages according to questions. This dataset, Economic trends, is a corpus of periodic reports on major economic indicators in Korea, intended for complex-document understanding tasks.",
        reference="https://github.com/whybe-choi/kovidore-data-generator",
        dataset={
            "path": "whybe-choi/kovidore-v2-economic-mteb",
            "revision": "0189c26211290a902cd9d41a0db932808a54c0a8",
        },
        type="DocumentUnderstanding",
        category="t2i",
        eval_splits=["test"],
        eval_langs=["kor-Hang"],
        main_score="ndcg_at_10",
        date=("2025-12-21", "2026-01-06"),
        domains=["Social"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation="""
@misc{choi2026kovidorev2,
  author = {Yongbin Choi},
  note = {A benchmark for evaluating Korean vision document retrieval with multi-page reasoning queries in practical domains},
  title = {KoViDoRe v2: a comprehensive evaluation of vision document retrieval for enterprise use-cases},
  url = {https://github.com/whybe-choi/kovidore-data-generator},
  year = {2026},
}
""",
        prompt={"query": "Find a screenshot that is relevant to the user's question."},
    )


class KoVidore2EnergyRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="KoVidore2EnergyRetrieval",
        description="Retrieve associated pages according to questions. This dataset, Energy, is a corpus of reports on energy market trends, policy planning, and industry statistics, intended for complex-document understanding tasks.",
        reference="https://github.com/whybe-choi/kovidore-data-generator",
        dataset={
            "path": "whybe-choi/kovidore-v2-energy-mteb",
            "revision": "8c09a3d22b1fa3a7f5e815e9521da9b048754211",
        },
        type="DocumentUnderstanding",
        category="t2i",
        eval_splits=["test"],
        eval_langs=["kor-Hang"],
        main_score="ndcg_at_10",
        date=("2025-12-21", "2026-01-06"),
        domains=["Social"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation="""
@misc{choi2026kovidorev2,
  author = {Yongbin Choi},
  note = {A benchmark for evaluating Korean vision document retrieval with multi-page reasoning queries in practical domains},
  title = {KoViDoRe v2: a comprehensive evaluation of vision document retrieval for enterprise use-cases},
  url = {https://github.com/whybe-choi/kovidore-data-generator},
  year = {2026},
}
""",
        prompt={"query": "Find a screenshot that is relevant to the user's question."},
    )


class KoVidore2HrRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="KoVidore2HrRetrieval",
        description="Retrieve associated pages according to questions. This dataset, HR, is a corpus of reports on workforce outlook and employment policy in korea, intended for complex-document understanding tasks.",
        reference="https://github.com/whybe-choi/kovidore-data-generator",
        dataset={
            "path": "whybe-choi/kovidore-v2-hr-mteb",
            "revision": "d9432c782a9a3e2eed064f6fac08b4c967d92b99",
        },
        type="DocumentUnderstanding",
        category="t2i",
        eval_splits=["test"],
        eval_langs=["kor-Hang"],
        main_score="ndcg_at_10",
        date=("2025-12-21", "2026-01-06"),
        domains=["Social"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation="""
@misc{choi2026kovidorev2,
  author = {Yongbin Choi},
  note = {A benchmark for evaluating Korean vision document retrieval with multi-page reasoning queries in practical domains},
  title = {KoViDoRe v2: a comprehensive evaluation of vision document retrieval for enterprise use-cases},
  url = {https://github.com/whybe-choi/kovidore-data-generator},
  year = {2026},
}
""",
        prompt={"query": "Find a screenshot that is relevant to the user's question."},
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class FSDnoisy18kAudioReranking(AbsTaskRetrieval):
    """FSDnoisy18k dataset adapted for audio reranking task.

    FSDnoisy18k is an audio dataset collected with the aim of fostering the investigation of label noise in
    sound event classification. It contains audio clips unequally distributed among 20 sound classes.
    The audio clips are of variable length and can contain multiple sound events.
    This version is adapted for audio reranking where given a query audio from one of the 20 sound classes,
    the task is to rank positive audio samples (same class) higher than negative samples (different classes).

    Each query has 4 positive examples (same sound class) and 16 negative examples
    (different sound classes), creating a challenging noisy audio retrieval scenario
    with 20 total candidates per query. The dataset contains 200 queries providing robust
    evaluation across all 20 sound event categories, including handling of label noise.
    """

    metadata = TaskMetadata(
        name="FSDnoisy18kAudioReranking",
        description="FSDnoisy18k sound event dataset adapted for audio reranking. Given a query audio with potential label noise, rank 4 relevant audio samples higher than 16 irrelevant ones from different sound classes. Contains 200 queries across 20 sound event categories.",
        reference="https://zenodo.org/record/2529934",
        dataset={
            "path": "mteb/FSDnoisy18kAudioReranking",
            "revision": "4ce2173166a65092eb745127fcc75c489ea3b859",
        },
        type="AudioReranking",
        category="a2a",
        modalities=["audio"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="map_at_1000",
        date=("2018-01-01", "2018-12-31"),
        domains=["AudioScene"],
        task_subtypes=["Environment Sound Reranking"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{fonseca2019fsdnoisy18k,
  author = {Fonseca, Eduardo and Plakal, Manoj and Ellis, Daniel P. W. and Font, Frederic and Favory, Xavier and Serra, Xavier},
  booktitle = {ICASSP 2019 - 2019 IEEE International Conference on Acoustics, Speech and Signal Processing (ICASSP)},
  organization = {IEEE},
  pages = {21--25},
  title = {Learning Sound Event Classifiers from Web Audio with Noisy Labels},
  year = {2019},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class TRECCOVID(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="TRECCOVID",
        description="TRECCOVID is an ad-hoc search challenge based on the COVID-19 dataset containing scientific articles related to the COVID-19 pandemic.",
        reference="https://ir.nist.gov/covidSubmit/index.html",
        dataset={
            "path": "mteb/trec-covid",
            "revision": "bb9466bac8153a0349341eb1b22e06409e78ef4e",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2020-01-01", "2021-12-31"),
        domains=["Medical", "Academic", "Written"],
        task_subtypes=["Article retrieval"],
        license="not specified",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{roberts2021searching,
  archiveprefix = {arXiv},
  author = {Kirk Roberts and Tasmeer Alam and Steven Bedrick and Dina Demner-Fushman and Kyle Lo and Ian Soboroff and Ellen Voorhees and Lucy Lu Wang and William R Hersh},
  eprint = {2104.09632},
  primaryclass = {cs.IR},
  title = {Searching for Scientific Evidence in a Pandemic: An Overview of TREC-COVID},
  year = {2021},
}
""",
        prompt={
            "query": "Given a query on COVID-19, retrieve documents that answer the query"
        },
    )



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class RonSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="RonSTS",
        dataset={
            "path": "mteb/RonSTS",
            "revision": "0e9021b8beb24b874230b4fc4e3d1612e72795e0",
        },
        description="High-quality Romanian translation of STSBenchmark.",
        reference="https://openreview.net/forum?id=JH61CD7afTv",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ron-Latn"],
        main_score="cosine_spearman",
        date=("2020-01-01", "2021-01-31"),
        domains=["News", "Social", "Web", "Written"],  # web for image captions
        task_subtypes=[],
        license="cc-by-4.0",  # not specified
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="machine-translated and verified",
        bibtex_citation=r"""
@inproceedings{dumitrescu2021liro,
  author = {Dumitrescu, Stefan Daniel and Rebeja, Petru and Lorincz, Beata and Gaman, Mihaela and Avram, Andrei and Ilie, Mihai and Pruteanu, Andrei and Stan, Adriana and Rosia, Lorena and Iacobescu, Cristina and others},
  booktitle = {Thirty-fifth Conference on Neural Information Processing Systems Datasets and Benchmarks Track (Round 1)},
  title = {LiRo: Benchmark and leaderboard for Romanian language tasks},
  year = {2021},
}
""",
    )

    min_score = 0
    max_score = 5



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.text.bitext_mining import AbsTaskBitextMining

_LANGUAGES = [
    "ben-Beng",
    "guj_Gujr",
    "hin_Deva",
    "kan_Knda",
    "mal_Mlym",
    "mar_Deva",
    "tam_Taml",
    "tel_Telu",
    "urd_Arab",
    "asm_Beng",
    "bho_Deva",
    "nep_Deva",
    "ory_Orya",
    "pan_Guru",
    "pus_Arab",
    "san-Deva",
    "awa_Deva",
    "bgc_Deva",
    "bod_Tibt",
    "boy_Deva",
    "gbm_Deva",
    "gom_Deva",
    "hne_Deva",
    "raj_Deva",
    "mai_Deva",
    "mni_Mtei",
    "mup_Deva",
    "mwr_Deva",
    "sat_Olck",
]

_ENG_LANGUAGE = ["eng-Latn"]

_CODE_MAPPING = {
    "ben": "bn",
    "guj": "gu",
    "hin": "hi",
    "kan": "kn",
    "mal": "ml",
    "mar": "mr",
    "tam": "ta",
    "tel": "te",
    "urd": "ur",
    "asm": "as",
    "bho": "bho",
    "nep": "ne",
    "ory": "or",
    "pan": "pa",
    "pus": "ps",
    "san": "sa",
    "awa": "awa",
    "bgc": "bgc",
    "bod": "bo",
    "boy": "brx",
    "gbm": "gbm",
    "gom": "gom",
    "hne": "hne",
    "raj": "hoj",
    "mai": "mai",
    "mni": "mni",
    "mup": "mup",
    "mwr": "mwr",
    "sat": "sat",
}

_SPLIT = ["validation", "test"]


def get_lang_pairs() -> dict[str, list[str]]:
    # add eng-> xx and xx -> eng lang pairs
    # Normalize language codes
    normalized_languages = [lang.replace("_", "-") for lang in _LANGUAGES]

    # Create dictionary for language pairs
    language_pairs = {}
    for lang in normalized_languages:
        lang_code = lang.split("-")[0]
        lang_to_eng_key = f"{lang_code}-eng"
        eng_to_lang_key = f"eng-{lang_code}"
        language_pairs[lang_to_eng_key] = [lang, _ENG_LANGUAGE[0]]
        language_pairs[eng_to_lang_key] = [_ENG_LANGUAGE[0], lang]

    return language_pairs


_LANGUAGES_MAPPING = get_lang_pairs()


class IndicGenBenchFloresBitextMining(AbsTaskBitextMining):
    metadata = TaskMetadata(
        name="IndicGenBenchFloresBitextMining",
        dataset={
            "path": "mteb/IndicGenBenchFloresBitextMining",
            "revision": "07dcc23c08a2540ba37ebe1e487da9dc497cc15c",
        },
        description="Flores-IN dataset is an extension of Flores dataset released as a part of the IndicGenBench by Google",
        reference="https://github.com/google-research-datasets/indic-gen-bench/",
        type="BitextMining",
        category="t2t",
        modalities=["text"],
        eval_splits=_SPLIT,
        eval_langs=_LANGUAGES_MAPPING,
        main_score="f1",
        date=("2023-10-01", "2024-05-01"),
        domains=["Web", "News", "Written"],
        task_subtypes=[],
        license="cc-by-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="human-translated and localized",
        bibtex_citation=r"""
@misc{singh2024indicgenbench,
  archiveprefix = {arXiv},
  author = {Harman Singh and Nitish Gupta and Shikhar Bharadwaj and Dinesh Tewari and Partha Talukdar},
  eprint = {2404.16816},
  primaryclass = {cs.CL},
  title = {IndicGenBench: A Multilingual Benchmark to Evaluate Generation Capabilities of LLMs on Indic Languages},
  year = {2024},
}
""",
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

from .facebookai import XLMR_LANGUAGES

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType


METACLIP2_CITATION = """@article{xu2025metaclip2,
  title={MetaCLIP 2: A Worldwide Scaling Recipe},
  author={Xu, Hu and Xie, Saining and Ghosh, Gargi and Kira, Zsolt and Darrell, Trevor},
  journal={arXiv preprint arXiv:2507.22062},
  year={2025}
}"""


class MetaClip2Model(AbsEncoder):
    """Wrapper for MetaCLIP 2 models.

    MetaCLIP 2 is a multilingual vision-language model that uses the mT5 tokenizer
    for worldwide language support.
    """

    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        **kwargs: Any,
    ):
        from transformers import AutoModel, AutoProcessor

        self.model_name = model_name
        self.device = device
        self.model = AutoModel.from_pretrained(
            model_name,
            revision=revision,
        ).to(self.device)
        self.processor = AutoProcessor.from_pretrained(
            model_name,
            revision=revision,
        )

    def get_text_embeddings(
        self,
        texts: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        all_text_embeddings = []

        with torch.no_grad():
            for batch in tqdm(
                texts, disable=not show_progress_bar, desc="Text Encoding"
            ):
                inputs = self.processor(
                    text=batch["text"],
                    return_tensors="pt",
                    padding=True,
                    truncation=True,
                )
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                text_outputs = self.model.get_text_features(**inputs)
                # MetaCLIP 2 returns BaseModelOutputWithPooling, extract pooler_output
                if hasattr(text_outputs, "pooler_output"):
                    text_outputs = text_outputs.pooler_output
                all_text_embeddings.append(text_outputs.cpu())

        all_text_embeddings = torch.cat(all_text_embeddings, dim=0)
        return all_text_embeddings

    @torch.no_grad()
    def get_image_embeddings(
        self,
        images: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        all_image_embeddings = []

        for batch in tqdm(images, disable=not show_progress_bar, desc="Image Encoding"):
            inputs = self.processor(
                images=batch["image"],
                return_tensors="pt",
                padding=True,
            )
            inputs = {k: v.to(self.device) for k, v in inputs.items()}
            image_outputs = self.model.get_image_features(**inputs)
            # MetaCLIP 2 returns BaseModelOutputWithPooling, extract pooler_output
            if hasattr(image_outputs, "pooler_output"):
                image_outputs = image_outputs.pooler_output
            all_image_embeddings.append(image_outputs.cpu())

        all_image_embeddings = torch.cat(all_image_embeddings, dim=0)
        return all_image_embeddings

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        image_embeddings = None
        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "image" in inputs.dataset.features:
            image_embeddings = self.get_image_embeddings(inputs, **kwargs)

        if text_embeddings is not None and image_embeddings is not None:
            if len(text_embeddings) != len(image_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + image_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif image_embeddings is not None:
            return image_embeddings
        raise ValueError


metaclip2_mt5_worldwide_b32 = ModelMeta(
    loader=MetaClip2Model,
    name="facebook/metaclip-2-mt5-worldwide-b32",
    model_type=["dense"],
    languages=XLMR_LANGUAGES,
    revision="fbbce525749bfc4a54b932bafe85313ee889d98f",
    release_date="2025-11-12",
    modalities=["image", "text"],
    n_parameters=253980417,
    n_embedding_parameters=128_057_344,
    memory_usage_mb=969,
    max_tokens=77,
    embed_dim=512,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="https://github.com/facebookresearch/MetaCLIP",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/metaclip-2-mt5-worldwide-b32",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets={"CommonCrawl"},
    citation=METACLIP2_CITATION,
)



from .argu_ana_nl_retrieval import ArguAnaNL, ArguAnaNLv2
from .bbsard_nl_retrieval import BBSARDNLRetrieval
from .climate_fevernl_retrieval import ClimateFEVERNL
from .cqa_dupstack_android_nl_retrieval import CQADupstackAndroidNLRetrieval
from .cqa_dupstack_english_nl_retrieval import CQADupstackEnglishNLRetrieval
from .cqa_dupstack_gaming_nl_retrieval import CQADupstackGamingNLRetrieval
from .cqa_dupstack_gis_nl_retrieval import CQADupstackGisNLRetrieval
from .cqa_dupstack_mathematica_nl_retrieval import CQADupstackMathematicaNLRetrieval
from .cqa_dupstack_physics_nl_retrieval import CQADupstackPhysicsNLRetrieval
from .cqa_dupstack_programmers_nl_retrieval import CQADupstackProgrammersNLRetrieval
from .cqa_dupstack_stats_nl_retrieval import CQADupstackStatsNLRetrieval
from .cqa_dupstack_tex_nl_retrieval import CQADupstackTexNLRetrieval
from .cqa_dupstack_unix_nl_retrieval import CQADupstackUnixNLRetrieval
from .cqa_dupstack_webmasters_nl_retrieval import CQADupstackWebmastersNLRetrieval
from .cqa_dupstack_wordpress_nl_retrieval import CQADupstackWordpressNLRetrieval
from .db_pedia_nl_retrieval import DBPediaNL
from .dutch_news_articles_retrieval import DutchNewsArticlesRetrieval
from .fevernl_retrieval import FEVERNL
from .fi_qa2018_nl_retrieval import FiQA2018NL
from .hotpot_qanl_retrieval import HotpotQANL
from .legal_qa_nl_retrieval import LegalQANLRetrieval
from .mmarconl_retrieval import MMMARCONL
from .nf_corpus_nl_retrieval import NFCorpusNL, NFCorpusNLv2
from .nqnl_retrieval import NQNL
from .open_tender_retrieval import OpenTenderRetrieval
from .quora_nl_retrieval import QuoraNLRetrieval
from .sci_fact_nl_retrieval import SciFactNL, SciFactNLv2
from .scidocsnl_retrieval import SCIDOCSNL, SCIDOCSNLv2
from .touche2020_nl_retrieval import Touche2020NL
from .treccovidnl_retrieval import TRECCOVIDNL
from .vabb_retrieval import VABBRetrieval

__all__ = [
    "FEVERNL",
    "MMMARCONL",
    "NQNL",
    "SCIDOCSNL",
    "TRECCOVIDNL",
    "ArguAnaNL",
    "ArguAnaNLv2",
    "BBSARDNLRetrieval",
    "CQADupstackAndroidNLRetrieval",
    "CQADupstackEnglishNLRetrieval",
    "CQADupstackGamingNLRetrieval",
    "CQADupstackGisNLRetrieval",
    "CQADupstackMathematicaNLRetrieval",
    "CQADupstackPhysicsNLRetrieval",
    "CQADupstackProgrammersNLRetrieval",
    "CQADupstackStatsNLRetrieval",
    "CQADupstackTexNLRetrieval",
    "CQADupstackUnixNLRetrieval",
    "CQADupstackWebmastersNLRetrieval",
    "CQADupstackWordpressNLRetrieval",
    "ClimateFEVERNL",
    "DBPediaNL",
    "DutchNewsArticlesRetrieval",
    "FiQA2018NL",
    "HotpotQANL",
    "LegalQANLRetrieval",
    "NFCorpusNL",
    "NFCorpusNLv2",
    "OpenTenderRetrieval",
    "QuoraNLRetrieval",
    "SCIDOCSNLv2",
    "SciFactNL",
    "SciFactNLv2",
    "Touche2020NL",
    "VABBRetrieval",
]



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class IEMOCAPGenderClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="IEMOCAPGender",
        description="Classification of speech samples by speaker gender (male/female) from the IEMOCAP database of interactive emotional dyadic conversations.",
        reference="https://doi.org/10.1007/s10579-008-9076-6",
        dataset={
            "path": "mteb/iemocap",
            "revision": "6d4225271da423e791e16770d335cfa351cdf88e",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2008-01-01", "2008-12-31"),
        domains=["Spoken", "Speech"],
        task_subtypes=["Gender Classification"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="expert-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="created",
        bibtex_citation=r"""
@article{busso2008iemocap,
  author = {Busso, Carlos and Bulut, Murtaza and Lee, Chi-Chun and Kazemzadeh, Abe and Mower, Emily and Kim, Samuel and Chang, Jeannette N and Lee, Sungbok and Narayanan, Shrikanth S},
  journal = {Language resources and evaluation},
  number = {4},
  pages = {335--359},
  publisher = {Springer},
  title = {IEMOCAP: Interactive emotional dyadic motion capture database},
  volume = {42},
  year = {2008},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "gender_id"

    is_cross_validation: bool = True

    def dataset_transform(self):
        # Define label mapping
        label2id = {"Female": 0, "Male": 1}

        # Apply transformation to all dataset splits
        for split in self.dataset:
            # Define transform function to add numeric labels
            def add_gender_id(example):
                example["gender_id"] = label2id[example["gender"]]
                return example

            print(f"Converting gender labels to numeric IDs for split '{split}'...")
            self.dataset[split] = self.dataset[split].map(add_gender_id)



from __future__ import annotations

import warnings
from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm
from transformers import (
    SpeechT5ForSpeechToText,
    SpeechT5ForTextToSpeech,
    SpeechT5Processor,
)

from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput, TextInput


class SpeechT5Audio(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_s: float = 30.0,
        **kwargs: Any,
    ):
        self.device = device
        self.max_audio_length_s = max_audio_length_s

        self.asr_processor = SpeechT5Processor.from_pretrained(
            "microsoft/speecht5_asr",
            revision=revision,
        )
        self.asr_model = SpeechT5ForSpeechToText.from_pretrained(
            "microsoft/speecht5_asr",
            revision=revision,
        ).to(self.device)
        self.asr_model.eval()

        self.sampling_rate = self.asr_processor.feature_extractor.sampling_rate

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        import torchaudio

        all_embeddings = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            batch_arrays = []
            for a in batch["audio"]:
                array = torch.tensor(a["array"], dtype=torch.float32)
                sr = a["sampling_rate"]
                if sr is None:
                    warnings.warn(
                        f"No sampling_rate provided for an audio sample. "
                        f"Assuming {self.sampling_rate} Hz (model default)."
                    )
                    sr = self.sampling_rate

                if sr != self.sampling_rate:
                    resampler = torchaudio.transforms.Resample(
                        orig_freq=sr, new_freq=self.sampling_rate
                    )
                    array = resampler(array)
                batch_arrays.append(array.numpy())

            if batch_arrays[0].ndim == 0:
                batch_arrays = [x.reshape(-1) for x in batch_arrays]
            elif batch_arrays[0].ndim > 1:
                batch_arrays = [x.reshape(x.size(0), -1) for x in batch_arrays]

            features = self.asr_processor(
                audio=batch_arrays,
                sampling_rate=self.sampling_rate,
                return_tensors="pt",
                padding="longest",
                truncation=True,
                max_length=int(self.max_audio_length_s * self.sampling_rate),
                return_attention_mask=True,
            ).to(self.device)

            with torch.no_grad():
                outputs = self.asr_model.speecht5.encoder(
                    input_values=features.input_values,
                    attention_mask=features.attention_mask,
                )
                last_hidden = outputs.last_hidden_state

                # Apply attention-masked pooling to exclude padding tokens
                batch_size, hidden_seq_len, hidden_size = last_hidden.shape
                device = last_hidden.device

                # Calculate proper hidden lengths based on input attention mask
                input_lengths = features.attention_mask.sum(dim=1)
                downsample_ratio = features.input_values.shape[1] / hidden_seq_len
                hidden_lengths = (input_lengths.float() / downsample_ratio).long()
                hidden_lengths = torch.clamp(hidden_lengths, min=0, max=hidden_seq_len)

                # Create attention mask for hidden states
                seq_range = torch.arange(hidden_seq_len, device=device).unsqueeze(0)
                hidden_attention_mask = (seq_range < hidden_lengths.unsqueeze(1)).to(
                    last_hidden.dtype
                )

                # Apply masked mean pooling
                hidden_attention_mask = hidden_attention_mask.unsqueeze(-1)
                masked_embeddings = last_hidden * hidden_attention_mask
                valid_tokens = hidden_attention_mask.sum(dim=1)
                embeddings = masked_embeddings.sum(dim=1) / valid_tokens.clamp(min=1e-9)

                all_embeddings.append(embeddings.cpu())

        return torch.cat(all_embeddings, dim=0)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if "audio" not in inputs.dataset.features:
            raise ValueError("ASTWrapper only supports audio inputs.")
        return self.get_audio_embeddings(inputs, **kwargs)


class SpeechT5Text(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        **kwargs: Any,
    ):
        self.device = device
        self.tts_processor = SpeechT5Processor.from_pretrained(
            "microsoft/speecht5_tts",
            revision=revision,
        )
        self.tts_model = SpeechT5ForTextToSpeech.from_pretrained(
            "microsoft/speecht5_tts",
            revision=revision,
        ).to(self.device)
        self.tts_model.eval()

    def get_text_embeddings(
        self,
        inputs: DataLoader[TextInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        """Get text embeddings using the text encoder."""
        all_embeddings = []

        for batch in tqdm(
            inputs, disable=not show_progress_bar, desc="Processing text batches"
        ):
            texts = batch["text"]

            # Process text through tokenizer
            tokenized = self.tts_processor(
                text=texts,
                return_tensors="pt",
                padding="longest",
                truncation=True,
            ).to(self.device)

            with torch.no_grad():
                outputs = self.tts_model.speecht5.encoder(
                    input_values=tokenized["input_ids"],
                    attention_mask=tokenized["attention_mask"],
                )

                last_hidden = outputs.last_hidden_state

                # Apply attention-masked pooling to exclude padding tokens
                attention_mask = (
                    tokenized["attention_mask"].unsqueeze(-1).to(last_hidden.dtype)
                )
                masked_embeddings = last_hidden * attention_mask
                valid_tokens = attention_mask.sum(dim=1)
                embeddings = masked_embeddings.sum(dim=1) / valid_tokens.clamp(min=1e-9)

            all_embeddings.append(embeddings.cpu())

        return torch.cat(all_embeddings, dim=0)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if "text" not in inputs.dataset.features:
            raise ValueError("ASTWrapper only supports audio inputs.")
        return self.get_text_embeddings(inputs, **kwargs)


class SpeechT2Multimodal(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_s: float = 30.0,
        **kwargs: Any,
    ):
        # Revision is combined as "asr_revision-tts_revision"
        asr_revision, tts_revision = revision.split("-")

        self.asr_encoder = SpeechT5Audio(
            model_name=model_name,
            revision=asr_revision,
            device=device,
            max_audio_length_s=max_audio_length_s,
            **kwargs,
        )
        self.tts_encoder = SpeechT5Text(
            model_name=model_name,
            revision=tts_revision,
            device=device,
            **kwargs,
        )

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        audio_embeddings = None
        if "text" in inputs.dataset.features:
            text_embeddings = self.tts_encoder.encode(
                inputs,
                task_metadata=task_metadata,
                hf_split=hf_split,
                hf_subset=hf_subset,
                prompt_type=prompt_type,
                **kwargs,
            )
        if "audio" in inputs.dataset.features:
            audio_embeddings = self.asr_encoder.encode(
                inputs,
                task_metadata=task_metadata,
                hf_split=hf_split,
                hf_subset=hf_subset,
                prompt_type=prompt_type,
                **kwargs,
            )

        if text_embeddings is not None and audio_embeddings is not None:
            if len(text_embeddings) != len(audio_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + audio_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif audio_embeddings is not None:
            return audio_embeddings
        raise ValueError


# ASR model - Optimized for Speech Recognition tasks
speecht5_asr = ModelMeta(
    loader=SpeechT5Audio,
    name="microsoft/speecht5_asr",
    languages=["eng-Latn"],
    open_weights=True,
    revision="53615c10408485422e09a12cda191a747f4bbe34",
    release_date="2022-05-16",
    max_tokens=None,
    n_parameters=151_575_936,
    memory_usage_mb=578,
    embed_dim=768,
    license="mit",
    reference="https://huggingface.co/microsoft/speecht5_asr",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code="https://github.com/microsoft/SpeechT5",
    public_training_data="https://www.openslr.org/12",
    training_datasets=set(),  # {"LibriSpeech": ["train"]},
    modalities=["audio"],
    citation="""
@misc{ao2022speecht5unifiedmodalencoderdecoderpretraining,
      title={SpeechT5: Unified-Modal Encoder-Decoder Pre-Training for Spoken Language Processing},
      author={Junyi Ao and Rui Wang and Long Zhou and Chengyi Wang and Shuo Ren and Yu Wu and Shujie Liu and Tom Ko and Qing Li and Yu Zhang and Zhihua Wei and Yao Qian and Jinyu Li and Furu Wei},
      year={2022},
      eprint={2110.07205},
      archivePrefix={arXiv},
      primaryClass={eess.AS},
      url={https://arxiv.org/abs/2110.07205},
}""",
)

# TTS model - Optimized for Text-to-Speech tasks
speecht5_tts = ModelMeta(
    loader=SpeechT5Text,
    name="microsoft/speecht5_tts",
    languages=["eng-Latn"],
    open_weights=True,
    revision="30fcde30f19b87502b8435427b5f5068e401d5f6",
    release_date="2022-05-16",
    max_tokens=None,
    n_parameters=146_335_465,
    memory_usage_mb=558,
    embed_dim=768,
    license="mit",
    reference="https://huggingface.co/microsoft/speecht5_tts",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code="https://github.com/microsoft/SpeechT5",
    public_training_data="https://www.openslr.org/12",
    training_datasets=set(),  # {"LibriTTS": ["train"]},
    modalities=["text"],
    citation="""
@misc{ao2022speecht5unifiedmodalencoderdecoderpretraining,
      title={SpeechT5: Unified-Modal Encoder-Decoder Pre-Training for Spoken Language Processing},
      author={Junyi Ao and Rui Wang and Long Zhou and Chengyi Wang and Shuo Ren and Yu Wu and Shujie Liu and Tom Ko and Qing Li and Yu Zhang and Zhihua Wei and Yao Qian and Jinyu Li and Furu Wei},
      year={2022},
      eprint={2110.07205},
      archivePrefix={arXiv},
      primaryClass={eess.AS},
      url={https://arxiv.org/abs/2110.07205},
}""",
)

# This model meta is for a multimodal model that combined from asr and tts.
speecht5_multimodal = ModelMeta(
    loader=SpeechT2Multimodal,
    name="microsoft/speecht5_multimodal",
    languages=["eng-Latn"],
    open_weights=True,
    revision="53615c10408485422e09a12cda191a747f4bbe34-30fcde30f19b87502b8435427b5f5068e401d5f6",
    release_date="2022-05-16",
    max_tokens=None,
    n_parameters=297_911_401,  # Combined ASR + TTS parameters
    memory_usage_mb=1136,  # Combined memory usage
    embed_dim=768,
    license="mit",
    reference="https://huggingface.co/microsoft/speecht5_asr",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code="https://github.com/microsoft/SpeechT5",
    public_training_data="http://www.festvox.org/cmu_arctic/",
    training_datasets=set(),
    modalities=["audio", "text"],
    citation="""
@misc{ao2022speecht5unifiedmodalencoderdecoderpretraining,
      title={SpeechT5: Unified-Modal Encoder-Decoder Pre-Training for Spoken Language Processing},
      author={Junyi Ao and Rui Wang and Long Zhou and Chengyi Wang and Shuo Ren and Yu Wu and Shujie Liu and Tom Ko and Qing Li and Yu Zhang and Zhihua Wei and Yao Qian and Jinyu Li and Furu Wei},
      year={2022},
      eprint={2110.07205},
      archivePrefix={arXiv},
      primaryClass={eess.AS},
      url={https://arxiv.org/abs/2110.07205},
}""",
)



from .wisesight_sentiment_classification import (
    WisesightSentimentClassification,
    WisesightSentimentClassificationV2,
)
from .wongnai_reviews_classification import WongnaiReviewsClassification

__all__ = [
    "WisesightSentimentClassification",
    "WisesightSentimentClassificationV2",
    "WongnaiReviewsClassification",
]



from mteb.models import ModelMeta, sentence_transformers_loader

yuan_emb_zh_datasets = {
    "CMedQAv2-reranking",
    "DuRetrieval",
    "MMarcoReranking",
    "T2Reranking",
    "T2Retrieval",
}

# not in mteb
# "Multi-CPR":"http://github.com/Alibaba-NLP/Multi-CPR",

yuan_embedding_2_zh = ModelMeta(
    name="IEITYuan/Yuan-embedding-2.0-zh",
    model_type=["dense"],
    loader=sentence_transformers_loader,
    languages=["zho-Hans"],
    open_weights=True,
    revision="b5ebcace6f4fc6e5a4d1852557eb2dc2d1040cee",
    release_date="2025-11-24",
    n_parameters=327359232,
    n_embedding_parameters=21_635_072,
    memory_usage_mb=1242,
    embed_dim=1792,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/IEITYuan/Yuan-embedding-2.0-zh",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=yuan_emb_zh_datasets,
)



from mteb.models.model_meta import ModelMeta
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

greennode_embedding_large_vn_v1_training_data = {
    "GreenNodeTableMarkdownRetrieval",
}

greennode_embedding_large_vn_v1 = ModelMeta(
    name="GreenNode/GreenNode-Embedding-Large-VN-V1",
    model_type=["dense"],
    revision="660def1f6e1c8ecdf39f6f9c95829e3cf0cef837",
    release_date="2024-04-11",
    languages=[
        "vie-Latn",
    ],
    loader=sentence_transformers_loader,
    open_weights=True,
    n_parameters=567754752,
    n_embedding_parameters=256_002_048,
    memory_usage_mb=2167,
    embed_dim=1024,
    license="cc-by-4.0",
    max_tokens=8194,
    reference="https://huggingface.co/GreenNode/GreenNode-Embedding-Large-VN-V1",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/GreenNode/GreenNode-Table-Markdown-Retrieval-VN",
    training_datasets=greennode_embedding_large_vn_v1_training_data,
    adapted_from="BAAI/bge-m3",
)

greennode_embedding_large_vn_mixed_v1 = ModelMeta(
    name="GreenNode/GreenNode-Embedding-Large-VN-Mixed-V1",
    model_type=["dense"],
    revision="1d3dddb3862292dab4bd3eddf0664c0335ad5843",
    release_date="2024-04-11",
    languages=[
        "vie-Latn",
    ],
    loader=sentence_transformers_loader,
    open_weights=True,
    n_parameters=567754752,
    n_embedding_parameters=256_002_048,
    memory_usage_mb=2167,
    embed_dim=1024,
    license="cc-by-4.0",
    max_tokens=8194,
    reference="https://huggingface.co/GreenNode/GreenNode-Embedding-Large-VN-Mixed-V1",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/GreenNode/GreenNode-Table-Markdown-Retrieval-VN",
    training_datasets=greennode_embedding_large_vn_v1_training_data,
    adapted_from="BAAI/bge-m3",
)

greennode_embedding_kalm_mini_instruct_vn_v1 = ModelMeta(
    name="GreenNode/GreenNode-Embedding-KaLM-Mini-Instruct-VN-V1",
    model_type=["dense"],
    revision="c123a4b0ef40ed847cb5122ff2c70ffc92129f3a",
    release_date="2026-02-26",
    languages=[
        "vie-Latn",
    ],
    loader=sentence_transformers_loader,
    open_weights=True,
    n_parameters=494032768,
    n_embedding_parameters=136_134_656,
    memory_usage_mb=1885,
    embed_dim=896,
    license="cc-by-4.0",
    max_tokens=32768,
    reference="https://huggingface.co/GreenNode/GreenNode-Embedding-KaLM-Mini-Instruct-VN-V1",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/GreenNode/GreenNode-Table-Markdown-Retrieval-VN",
    training_datasets=greennode_embedding_large_vn_v1_training_data,
    adapted_from="KaLM-Embedding/KaLM-embedding-multilingual-mini-instruct-v2.5",
)

greennode_embedding_e5_large_vn_v1 = ModelMeta(
    name="GreenNode/GreenNode-Embedding-E5-Large-VN-V1",
    model_type=["dense"],
    revision="a15ad86a2a4c80f168210a17cc5d540a52354113",
    release_date="2026-02-26",
    languages=[
        "vie-Latn",
    ],
    loader=sentence_transformers_loader,
    open_weights=True,
    n_parameters=560_000_000,
    n_embedding_parameters=256_002_048,
    memory_usage_mb=2136,
    embed_dim=1024,
    license="cc-by-4.0",
    max_tokens=512,
    reference="https://huggingface.co/GreenNode/GreenNode-Embedding-E5-Large-VN-V1",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/GreenNode/GreenNode-Table-Markdown-Retrieval-VN",
    training_datasets=greennode_embedding_large_vn_v1_training_data,
    adapted_from="intfloat/multilingual-e5-large",
)

aiteamvn_vietnamese_embeddings = ModelMeta(
    name="AITeamVN/Vietnamese_Embedding",
    model_type=["dense"],
    revision="dea33aa1ab339f38d66ae0a40e6c40e0a9249568",
    release_date="2024-03-17",
    languages=[
        "vie-Latn",
    ],
    loader=sentence_transformers_loader,
    open_weights=True,
    n_parameters=567754752,
    n_embedding_parameters=256_002_048,
    memory_usage_mb=2166,
    embed_dim=1024,
    license="cc-by-4.0",
    max_tokens=8194,
    reference="https://huggingface.co/AITeamVN/Vietnamese_Embedding",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    adapted_from="BAAI/bge-m3",
    citation="""@misc{Vietnamese_Embedding,
  title={Vietnamese_Embedding: Embedding model in Vietnamese language.},
  author={Nguyen Nho Trung, Nguyen Nhat Quang, Nguyen Van Huy},
  year={2025},
  publisher={Huggingface},
}""",
)


contextboxai_halong_embedding = ModelMeta(
    name="contextboxai/halong_embedding",
    model_type=["dense"],
    revision="b57776031035f70ed2030d2e35ecc533eb0f8f71",
    release_date="2024-07-06",
    languages=[
        "vie-Latn",
    ],
    loader=sentence_transformers_loader,
    use_instructions=False,
    open_weights=True,
    n_parameters=278043648,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=1061,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=514,
    reference="https://huggingface.co/contextboxai/halong_embedding",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    adapted_from="intfloat/multilingual-e5-base",
    citation="""@misc{HalongEmbedding,
  title={HalongEmbedding: A Vietnamese Text Embedding},
  author={Ngo Hieu},
  year={2024},
  publisher={Huggingface},
}""",
)

sup_simcse_vietnamese_phobert_base_ = ModelMeta(
    name="VoVanPhuc/sup-SimCSE-VietNamese-phobert-base",
    model_type=["dense"],
    revision="608779b86741a8acd8c8d38132974ff04086b138",
    release_date="2021-05-26",
    languages=[
        "vie-Latn",
    ],
    loader=sentence_transformers_loader,
    use_instructions=False,
    open_weights=True,
    n_parameters=134998272,
    n_embedding_parameters=49_152_768,
    memory_usage_mb=517,
    max_tokens=256,
    embed_dim=768,
    license="apache-2.0",
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "Transformers", "safetensors"],
    reference="https://huggingface.co/VoVanPhuc/sup-SimCSE-VietNamese-phobert-base",
    similarity_fn_name="cosine",
    training_datasets=None,
    citation="""@article{gao2021simcse,
   title={{SimCSE}: Simple Contrastive Learning of Sentence Embeddings},
   author={Gao, Tianyu and Yao, Xingcheng and Chen, Danqi},
   journal={arXiv preprint arXiv:2104.08821},
   year={2021}
}

@inproceedings{phobert,
title     = {{PhoBERT: Pre-trained language models for Vietnamese}},
author    = {Dat Quoc Nguyen and Anh Tuan Nguyen},
booktitle = {Findings of the Association for Computational Linguistics: EMNLP 2020},
year      = {2020},
pages     = {1037--1042}
}""",
)

bkai_foundation_models_vietnamese_bi_encoder = ModelMeta(
    name="bkai-foundation-models/vietnamese-bi-encoder",
    model_type=["dense"],
    revision="84f9d9ada0d1a3c37557398b9ae9fcedcdf40be0",
    release_date="2023-09-09",
    languages=[
        "vie-Latn",
    ],
    loader=sentence_transformers_loader,
    use_instructions=False,
    open_weights=True,
    n_parameters=134998272,
    n_embedding_parameters=49_152_768,
    memory_usage_mb=515,
    max_tokens=256,
    embed_dim=768,
    license="apache-2.0",
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/bkai-foundation-models/vietnamese-bi-encoder",
    similarity_fn_name="cosine",
    training_datasets=None,
    citation="""
      @article{duc2024towards,
    title={Towards Comprehensive Vietnamese Retrieval-Augmented Generation and Large Language Models},
    author={Nguyen Quang Duc, Le Hai Son, Nguyen Duc Nhan, Nguyen Dich Nhat Minh, Le Thanh Huong, Dinh Viet Sang},
    journal={arXiv preprint arXiv:2403.01616},
    year={2024}
  }""",
)



from mteb.abstasks import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata


class CREMADPairClassification(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="CREMADPairClassification",
        description="Classifying pairs as having same or different emotions in actor's voice recordings of text spoken in 6 different emotions",
        reference="https://pmc.ncbi.nlm.nih.gov/articles/PMC4313618/",
        dataset={
            "path": "mteb/CREMADPairClassification",
            "revision": "20701cbd51bf282719674c9f38ec91e698b62e48",
        },
        type="AudioPairClassification",
        category="a2a",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="max_ap",
        date=("2014-01-01", "2014-12-31"),
        domains=["Spoken"],
        task_subtypes=["Emotion classification"],
        license="http://opendatacommons.org/licenses/odbl/1.0/",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="created",
        bibtex_citation=r"""
@article{Cao2014-ih,
  author = {Cao, Houwei and Cooper, David G and Keutmann, Michael K and Gur,
Ruben C and Nenkova, Ani and Verma, Ragini},
  copyright = {https://ieeexplore.ieee.org/Xplorehelp/downloads/license-information/IEEE.html},
  journal = {IEEE Transactions on Affective Computing},
  keywords = {Emotional corpora; facial expression; multi-modal recognition;
voice expression},
  language = {en},
  month = oct,
  number = {4},
  pages = {377--390},
  publisher = {Institute of Electrical and Electronics Engineers (IEEE)},
  title = {{CREMA-D}: Crowd-sourced emotional multimodal actors dataset},
  volume = {5},
  year = {2014},
}
""",
    )

    input1_column_name: str = "audio1"
    input2_column_name: str = "audio2"
    label_column_name: str = "label"



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class LibriCount(AbsTaskClassification):
    metadata = TaskMetadata(
        name="LibriCount",
        description="Multiclass speaker count identification. Dataset contains audio recordings with between 0 to 10 speakers.",
        reference="https://huggingface.co/datasets/silky1708/LibriCount",
        dataset={
            "path": "mteb/libricount",
            "revision": "cc851c56e30dc5dde80c1823de96d52ca3cb2607",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2017-01-01", "2017-12-31"),
        domains=["Speech"],
        task_subtypes=["Speaker Count Identification"],
        license="cc-by-4.0",
        annotations_creators="algorithmic",  # VAD (Voice Activity Detection) algo
        dialect=[],
        modalities=["audio"],
        sample_creation="created",  # from LibriSpeech dataset
        bibtex_citation=r"""
@inproceedings{Stoter_2018,
  author = {Stoter, Fabian-Robert and Chakrabarty, Soumitro and Edler, Bernd and Habets, Emanuel A. P.},
  booktitle = {2018 IEEE International Conference on Acoustics, Speech and Signal Processing (ICASSP)},
  doi = {10.1109/icassp.2018.8462159},
  month = apr,
  pages = {436-440},
  publisher = {IEEE},
  title = {Classification vs. Regression in Supervised Learning for Single Channel Speaker Count Estimation},
  url = {http://dx.doi.org/10.1109/ICASSP.2018.8462159},
  year = {2018},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "label"

    is_cross_validation: bool = True



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class FinToxicityClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="FinToxicityClassification",
        description="This dataset is a DeepL -based machine translated version of the Jigsaw toxicity dataset for Finnish. The dataset is originally from a Kaggle competition https://www.kaggle.com/c/jigsaw-toxic-comment-classification-challenge/data. The original dataset poses a multi-label text classification problem and includes the labels identity_attack, insult, obscene, severe_toxicity, threat and toxicity. Here adapted for toxicity classification, which is the most represented class.",
        dataset={
            "path": "TurkuNLP/jigsaw_toxicity_pred_fi",
            "revision": "6e7340e6be87124f319e25290778760c14df64d3",
        },
        reference="https://aclanthology.org/2023.nodalida-1.68",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["fin-Latn"],
        main_score="f1",
        date=("2023-03-13", "2023-09-25"),
        domains=["News", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@inproceedings{eskelinen-etal-2023-toxicity,
  author = {Eskelinen, Anni  and
Silvala, Laura  and
Ginter, Filip  and
Pyysalo, Sampo  and
Laippala, Veronika},
  booktitle = {Proceedings of the 24th Nordic Conference on Computational Linguistics (NoDaLiDa)},
  month = may,
  title = {Toxicity Detection in {F}innish Using Machine Translation},
  year = {2023},
}
""",
        superseded_by="FinToxicityClassification.v2",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset = self.dataset.rename_column("label_toxicity", "label")
        remove_cols = [
            col
            for col in self.dataset["test"].column_names
            if col not in ["text", "label"]
        ]
        self.dataset = self.dataset.remove_columns(remove_cols)


class FinToxicityClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="FinToxicityClassification.v2",
        description="This dataset is a DeepL -based machine translated version of the Jigsaw toxicity dataset for Finnish. The dataset is originally from a Kaggle competition https://www.kaggle.com/c/jigsaw-toxic-comment-classification-challenge/data. The original dataset poses a multi-label text classification problem and includes the labels identity_attack, insult, obscene, severe_toxicity, threat and toxicity. Here adapted for toxicity classification, which is the most represented class. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        dataset={
            "path": "mteb/fin_toxicity",
            "revision": "1deba6e874be1d5632a4ac0d1fb71f4bc3dea0d6",
        },
        reference="https://aclanthology.org/2023.nodalida-1.68",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["fin-Latn"],
        main_score="f1",
        date=("2023-03-13", "2023-09-25"),
        domains=["News", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@inproceedings{eskelinen-etal-2023-toxicity,
  author = {Eskelinen, Anni  and
Silvala, Laura  and
Ginter, Filip  and
Pyysalo, Sampo  and
Laippala, Veronika},
  booktitle = {Proceedings of the 24th Nordic Conference on Computational Linguistics (NoDaLiDa)},
  month = may,
  title = {Toxicity Detection in {F}innish Using Machine Translation},
  year = {2023},
}
""",
        adapted_from=["FinToxicityClassification"],
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm
from transformers import HubertModel, Wav2Vec2FeatureExtractor

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_audio_dependencies
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput


class HubertWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_seconds: float = 30.0,
        **kwargs: Any,
    ):
        requires_audio_dependencies()
        self.model_name = model_name
        self.device = device
        self.max_audio_length_seconds = max_audio_length_seconds

        # HuBERT uses the same feature extractor as Wav2Vec2
        self.feature_extractor = Wav2Vec2FeatureExtractor.from_pretrained(model_name)
        self.model = HubertModel.from_pretrained(model_name, revision=revision).to(
            self.device
        )
        self.model.eval()
        self.sampling_rate = self.feature_extractor.sampling_rate

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        inputs.collate_fn = AudioCollator(self.sampling_rate)
        all_embeddings = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            audio_arrays = [audio["array"] for audio in batch["audio"]]

            feature_inputs = self.feature_extractor(
                audio_arrays,
                sampling_rate=self.sampling_rate,
                return_tensors="pt",
                padding="longest",
                truncation=True,
                max_length=int(self.max_audio_length_seconds * self.sampling_rate),
                return_attention_mask=True,
            ).to(self.device)

            with torch.no_grad():
                outputs = self.model(
                    feature_inputs.input_values,
                    attention_mask=feature_inputs.attention_mask,
                    output_hidden_states=True,
                )

                last_hidden_state = outputs.last_hidden_state

                # Apply attention-masked pooling to exclude padding tokens
                batch_size, hidden_seq_len, hidden_size = last_hidden_state.shape
                device = last_hidden_state.device

                # Calculate proper hidden lengths based on input attention mask
                input_lengths = feature_inputs.attention_mask.sum(dim=1)
                downsample_ratio = feature_inputs.input_values.shape[1] / hidden_seq_len
                hidden_lengths = (input_lengths.float() / downsample_ratio).long()
                hidden_lengths = torch.clamp(hidden_lengths, min=0, max=hidden_seq_len)

                # Create attention mask for hidden states
                seq_range = torch.arange(hidden_seq_len, device=device).unsqueeze(0)
                hidden_attention_mask = (seq_range < hidden_lengths.unsqueeze(1)).to(
                    last_hidden_state.dtype
                )

                # Apply masked mean pooling
                hidden_attention_mask = hidden_attention_mask.unsqueeze(-1)
                masked_embeddings = last_hidden_state * hidden_attention_mask
                valid_tokens = hidden_attention_mask.sum(dim=1)
                embeddings = masked_embeddings.sum(dim=1) / valid_tokens.clamp(min=1e-9)

                all_embeddings.append(embeddings.cpu().detach())

        return torch.cat(all_embeddings, dim=0).numpy()

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if "audio" not in inputs.dataset.features:
            raise ValueError("HubertWrapper only supports audio inputs.")
        return self.get_audio_embeddings(inputs, **kwargs)


# Base model
hubert_base = ModelMeta(
    loader=HubertWrapper,
    name="facebook/hubert-base-ls960",
    languages=["eng-Latn"],
    open_weights=True,
    revision="dba3bb02fda4248b6e082697eee756de8fe8aa8a",
    release_date="2021-06-14",  # Paper release date
    max_tokens=float("inf"),
    n_parameters=95_000_000,
    memory_usage_mb=360,
    embed_dim=768,
    license="mit",
    reference="https://huggingface.co/facebook/hubert-base-ls960",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code="https://github.com/pytorch/fairseq/tree/master/examples/hubert",
    public_training_data="https://www.openslr.org/12",  # Link to LibriSpeech Dataset
    training_datasets=set(),  # "LibriSpeech": ["train"]},
    modalities=["audio"],
    citation="""
@misc{hsu2021hubertselfsupervisedspeechrepresentation,
    title={HuBERT: Self-Supervised Speech Representation Learning by Masked Prediction of Hidden Units},
    author={Wei-Ning Hsu and Benjamin Bolte and Yao-Hung Hubert Tsai and Kushal Lakhotia and Ruslan Salakhutdinov and Abdelrahman Mohamed},
    year={2021},
    eprint={2106.07447},
    archivePrefix={arXiv},
    primaryClass={cs.CL},
    url={https://arxiv.org/abs/2106.07447},
}""",
)

# Fine-tuned large model
hubert_large_ft = ModelMeta(
    loader=HubertWrapper,
    name="facebook/hubert-large-ls960-ft",
    languages=["eng-Latn"],
    open_weights=True,
    revision="ece5fabbf034c1073acae96d5401b25be96709d8",
    release_date="2021-06-14",  # Paper release date
    max_tokens=float("inf"),
    n_parameters=317_000_000,
    memory_usage_mb=1203,
    embed_dim=1024,
    license="mit",
    reference="https://huggingface.co/facebook/hubert-large-ls960-ft",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code="https://github.com/pytorch/fairseq/tree/master/examples/hubert",
    public_training_data="https://www.openslr.org/12",  # Link to LibriSpeech Dataset
    training_datasets=set(),  # "LibriSpeech": ["train"]},
    modalities=["audio"],
    citation="""
@misc{hsu2021hubertselfsupervisedspeechrepresentation,
    title={HuBERT: Self-Supervised Speech Representation Learning by Masked Prediction of Hidden Units},
    author={Wei-Ning Hsu and Benjamin Bolte and Yao-Hung Hubert Tsai and Kushal Lakhotia and Ruslan Salakhutdinov and Abdelrahman Mohamed},
    year={2021},
    eprint={2106.07447},
    archivePrefix={arXiv},
    primaryClass={cs.CL},
    url={https://arxiv.org/abs/2106.07447},
}""",
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class PIQA(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="PIQA",
        description="Measuring the ability to retrieve the groundtruth answers to reasoning task queries on PIQA.",
        reference="https://arxiv.org/abs/1911.11641",
        dataset={
            "path": "mteb/PIQA",
            "revision": "aee23bb32dafbaddddf16cd5033d912693984d9a",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2020-01-01", "2020-12-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Reasoning as Retrieval"],
        license="afl-3.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{bisk2020piqa,
  author = {Bisk, Yonatan and Zellers, Rowan and Gao, Jianfeng and Choi, Yejin and others},
  booktitle = {Proceedings of the AAAI conference on artificial intelligence},
  number = {05},
  pages = {7432--7439},
  title = {Piqa: Reasoning about physical commonsense in natural language},
  volume = {34},
  year = {2020},
}

@article{xiao2024rar,
  author = {Xiao, Chenghao and Hudson, G Thomas and Moubayed, Noura Al},
  journal = {arXiv preprint arXiv:2404.06347},
  title = {RAR-b: Reasoning as Retrieval Benchmark},
  year = {2024},
}
""",
        prompt={"query": "Given the following goal, retrieve a possible solution."},
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class DanishPoliticalCommentsClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="DanishPoliticalCommentsClassification",
        dataset={
            "path": "mteb/DanishPoliticalCommentsClassification",
            "revision": "d743dcd5abb03d5ab357757a0e83522fc6696fcd",
        },
        description="A dataset of Danish political comments rated for sentiment",
        reference="https://huggingface.co/datasets/danish_political_comments",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["dan-Latn"],
        main_score="accuracy",
        date=(
            "2000-01-01",
            "2022-12-31",
        ),  # Estimated range for the collection of comments
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@techreport{SAMsentiment,
  author = {Mads Guldborg Kjeldgaard Kongsbak and Steffan Eybye Christensen and Lucas Høyberg Puvis~de~Chavannes and Peter Due Jensen},
  institution = {IT University of Copenhagen},
  title = {Sentiment Analysis Multitool, SAM},
  year = {2019},
}
""",
        prompt="Classify Danish political comments for sentiment",
        superseded_by="DanishPoliticalCommentsClassification.v2",
    )

    samples_per_label = 16


class DanishPoliticalCommentsClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="DanishPoliticalCommentsClassification.v2",
        dataset={
            "path": "mteb/danish_political_comments",
            "revision": "476a9e7327aba70ad3e97a169d7310b86be9b245",
        },
        description="A dataset of Danish political comments rated for sentiment This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://huggingface.co/datasets/danish_political_comments",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["dan-Latn"],
        main_score="accuracy",
        date=(
            "2000-01-01",
            "2022-12-31",
        ),  # Estimated range for the collection of comments
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@techreport{SAMsentiment,
  author = {Mads Guldborg Kjeldgaard Kongsbak and Steffan Eybye Christensen and Lucas Høyberg Puvis~de~Chavannes and Peter Due Jensen},
  institution = {IT University of Copenhagen},
  title = {Sentiment Analysis Multitool, SAM},
  year = {2019},
}
""",
        prompt="Classify Danish political comments for sentiment",
        adapted_from=["DanishPoliticalCommentsClassification"],
    )

    samples_per_label = 16



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class WikipediaChemFieldsClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="WikipediaChemFieldsClassification",
        description="ChemTEB evaluates the performance of text embedding models on chemical domain data.",
        reference="https://arxiv.org/abs/2412.00532",
        dataset={
            "path": "BASF-AI/WikipediaEZ10Classification",
            "revision": "a75fae77759acc115f015f2b856baa47776d733d",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2024-06-01", "2024-11-30"),
        domains=["Chemistry"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@article{kasmaee2024chemteb,
  author = {Kasmaee, Ali Shiraee and Khodadad, Mohammad and Saloot, Mohammad Arshi and Sherck, Nick and Dokas, Stephen and Mahyar, Hamidreza and Samiee, Soheila},
  journal = {arXiv preprint arXiv:2412.00532},
  title = {ChemTEB: Chemical Text Embedding Benchmark, an Overview of Embedding Models Performance \& Efficiency on a Specific Domain},
  year = {2024},
}
""",
        superseded_by="WikipediaChemFieldsClassification.v2",
    )


class WikipediaChemFieldsClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="WikipediaChemFieldsClassification.v2",
        description="ChemTEB evaluates the performance of text embedding models on chemical domain data. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://arxiv.org/abs/2412.00532",
        dataset={
            "path": "mteb/wikipedia_chem_fields",
            "revision": "70ff36ec5157fcc54c59f3b85d2b1fb232f8feec",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2024-06-01", "2024-11-30"),
        domains=["Chemistry"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@article{kasmaee2024chemteb,
  author = {Kasmaee, Ali Shiraee and Khodadad, Mohammad and Saloot, Mohammad Arshi and Sherck, Nick and Dokas, Stephen and Mahyar, Hamidreza and Samiee, Soheila},
  journal = {arXiv preprint arXiv:2412.00532},
  title = {ChemTEB: Chemical Text Embedding Benchmark, an Overview of Embedding Models Performance \& Efficiency on a Specific Domain},
  year = {2024},
}
""",
        adapted_from=["WikipediaChemFieldsClassification"],
    )
